# sage_setup: distribution = sagemath-categories
r"""
Graded bialgebras with basis
"""
#*****************************************************************************
#  Copyright (C) 2008 Teresa Gomez-Diaz (CNRS) <Teresa.Gomez-Diaz@univ-mlv.fr>
#                2011 Nicolas M. Thiery <nthiery at users.sf.net>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#                  http://www.gnu.org/licenses/
#******************************************************************************


def GradedBialgebrasWithBasis(base_ring):
    """
    The category of graded bialgebras with a distinguished basis

    EXAMPLES::

        sage: C = GradedBialgebrasWithBasis(QQ); C
        Join of Category of ...
        sage: sorted(C.super_categories(), key=str)
        [Category of bialgebras with basis over Rational Field,
         Category of graded algebras with basis over Rational Field,
         Category of graded coalgebras with basis over Rational Field]

    TESTS::

        sage: TestSuite(C).run()
    """
    from sage.categories.bialgebras_with_basis import BialgebrasWithBasis
    return BialgebrasWithBasis(base_ring).Graded()



# sage_setup: distribution = sagemath-categories
# sage.doctest: needs sage.graphs, sage.combinat
r"""
Posets
"""
# ****************************************************************************
#  Copyright (C) 2011 Nicolas M. Thiery <nthiery at users.sf.net>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#                  https://www.gnu.org/licenses/
# *****************************************************************************
from sage.misc.cachefunc import cached_method
from sage.misc.abstract_method import abstract_method
from sage.misc.lazy_import import LazyImport
from sage.categories.category import Category
from sage.categories.sets_cat import Sets


class Posets(Category):
    r"""
    The category of posets i.e. sets with a partial order structure.

    EXAMPLES::

        sage: Posets()
        Category of posets
        sage: Posets().super_categories()
        [Category of sets]
        sage: P = Posets().example(); P
        An example of a poset: sets ordered by inclusion

    The partial order is implemented by the mandatory method
    :meth:`~Posets.ParentMethods.le`::

        sage: x = P(Set([1,3])); y = P(Set([1,2,3]))
        sage: x, y
        ({1, 3}, {1, 2, 3})
        sage: P.le(x, y)
        True
        sage: P.le(x, x)
        True
        sage: P.le(y, x)
        False

    The other comparison methods are called
    :meth:`~Posets.ParentMethods.lt`, :meth:`~Posets.ParentMethods.ge`,
    :meth:`~Posets.ParentMethods.gt`, following Python's naming
    convention in :mod:`operator`. Default implementations are
    provided::

        sage: P.lt(x, x)
        False
        sage: P.ge(y, x)
        True

    Unless the poset is a facade (see :class:`Sets.Facade`), one can
    compare directly its elements using the usual Python operators::

        sage: D = Poset((divisors(30), attrcall("divides")), facade = False)
        sage: D(3) <= D(6)
        True
        sage: D(3) <= D(3)
        True
        sage: D(3) <= D(5)
        False
        sage: D(3) < D(3)
        False
        sage: D(10) >= D(5)
        True

    At this point, this has to be implemented by hand. Once
    :issue:`10130` will be resolved, this will be automatically
    provided by this category::

        sage: # not implemented
        sage: x < y
        True
        sage: x < x
        False
        sage: x <= x
        True
        sage: y >= x
        True

    .. SEEALSO:: :func:`Poset`, :class:`FinitePosets`, :class:`LatticePosets`

    TESTS::

        sage: C = Posets()
        sage: TestSuite(C).run()

    """
    @cached_method
    def super_categories(self):
        r"""
        Return a list of the (immediate) super categories of
        ``self``, as per :meth:`Category.super_categories`.

        EXAMPLES::

            sage: Posets().super_categories()
            [Category of sets]
        """
        return [Sets()]

    def example(self, choice=None):
        r"""
        Return examples of objects of ``Posets()``, as per
        :meth:`Category.example()
        <sage.categories.category.Category.example>`.

        EXAMPLES::

            sage: Posets().example()
            An example of a poset: sets ordered by inclusion

            sage: Posets().example("facade")
            An example of a facade poset:
             the positive integers ordered by divisibility
        """
        from sage.categories.examples.posets import FiniteSetsOrderedByInclusion, PositiveIntegersOrderedByDivisibilityFacade
        if choice == "facade":
            return PositiveIntegersOrderedByDivisibilityFacade()
        else:
            return FiniteSetsOrderedByInclusion()

    def __iter__(self):
        r"""
        Iterator over representatives of the isomorphism classes of
        posets with finitely many vertices.

        .. warning:: this feature may become deprecated, since it does
           of course not iterate through all posets.

        EXAMPLES::

            sage: P = Posets()
            sage: it = iter(P)
            sage: for _ in range(10): print(next(it))
            Finite poset containing 0 elements
            Finite poset containing 1 elements
            Finite poset containing 2 elements
            Finite poset containing 2 elements
            Finite poset containing 3 elements
            Finite poset containing 3 elements
            Finite poset containing 3 elements
            Finite poset containing 3 elements
            Finite poset containing 3 elements
            Finite poset containing 4 elements
        """
        from sage.combinat.posets.posets import FinitePosets_n
        n = 0
        while True:
            yield from FinitePosets_n(n)
            n += 1

    Finite = LazyImport('sage.categories.finite_posets', 'FinitePosets')

    class ParentMethods:

        @abstract_method
        def le(self, x, y):
            r"""
            Return whether `x \le y` in the poset ``self``.

            INPUT:

            - ``x``, ``y`` -- elements of ``self``.

            EXAMPLES::

                sage: D = Poset((divisors(30), attrcall("divides")))
                sage: D.le( 3, 6 )
                True
                sage: D.le( 3, 3 )
                True
                sage: D.le( 3, 5 )
                False
            """

        def lt(self, x, y):
            r"""
            Return whether `x < y` in the poset ``self``.

            INPUT:

            - ``x``, ``y`` -- elements of ``self``.

            This default implementation delegates the work to :meth:`le`.

            EXAMPLES::

                sage: D = Poset((divisors(30), attrcall("divides")))
                sage: D.lt( 3, 6 )
                True
                sage: D.lt( 3, 3 )
                False
                sage: D.lt( 3, 5 )
                False
            """
            return self.le(x,y) and x != y

        def ge(self, x, y):
            r"""
            Return whether `x \ge y` in the poset ``self``.

            INPUT:

            - ``x``, ``y`` -- elements of ``self``.

            This default implementation delegates the work to :meth:`le`.

            EXAMPLES::

                sage: D = Poset((divisors(30), attrcall("divides")))
                sage: D.ge( 6, 3 )
                True
                sage: D.ge( 3, 3 )
                True
                sage: D.ge( 3, 5 )
                False
            """
            return self.le(y,x)

        def gt(self, x, y):
            r"""
            Return whether `x > y` in the poset ``self``.

            INPUT:

            - ``x``, ``y`` -- elements of ``self``.

            This default implementation delegates the work to :meth:`lt`.

            EXAMPLES::

                sage: D = Poset((divisors(30), attrcall("divides")))
                sage: D.gt( 3, 6 )
                False
                sage: D.gt( 3, 3 )
                False
                sage: D.gt( 3, 5 )
                False
            """
            return self.lt(y,x)

        @abstract_method(optional=True)
        def upper_covers(self, x):
            r"""
            Return the upper covers of `x`, that is, the elements `y`
            such that `x<y` and there exists no `z` such that `x<z<y`.

            EXAMPLES::

                sage: D = Poset((divisors(30), attrcall("divides")))
                sage: D.upper_covers(3)
                [6, 15]
            """

        @abstract_method(optional=True)
        def lower_covers(self, x):
            r"""
            Return the lower covers of `x`, that is, the elements `y`
            such that `y<x` and there exists no `z` such that `y<z<x`.

            EXAMPLES::

                sage: D = Poset((divisors(30), attrcall("divides")))
                sage: D.lower_covers(15)
                [3, 5]
            """

        @abstract_method(optional=True)
        def order_ideal(self, elements):
            r"""
            Return the order ideal in ``self`` generated by the elements
            of an iterable ``elements``.

            A subset `I` of a poset is said to be an order ideal if, for
            any `x` in `I` and `y` such that `y \le x`, then `y` is in `I`.

            This is also called the lower set generated by these elements.

            EXAMPLES::

                sage: B = posets.BooleanLattice(4)
                sage: B.order_ideal([7,10])
                [0, 1, 2, 3, 4, 5, 6, 7, 8, 10]
            """

        @abstract_method(optional=True)
        def order_filter(self, elements):
            r"""
            Return the order filter generated by a list of elements.

            A subset `I` of a poset is said to be an order filter if, for
            any `x` in `I` and `y` such that `y \ge x`, then `y` is in `I`.

            This is also called the upper set generated by these elements.

            EXAMPLES::

                sage: B = posets.BooleanLattice(4)
                sage: B.order_filter([3,8])
                [3, 7, 8, 9, 10, 11, 12, 13, 14, 15]
            """

        def directed_subset(self, elements, direction):
            r"""
            Return the order filter or the order ideal generated by a
            list of elements.

            If ``direction`` is 'up', the order filter (upper set) is
            being returned.

            If ``direction`` is 'down', the order ideal (lower set) is
            being returned.

            INPUT:

            - elements -- a list of elements.

            - direction -- 'up' or 'down'.

            EXAMPLES::

                sage: B = posets.BooleanLattice(4)
                sage: B.directed_subset([3, 8], 'up')
                [3, 7, 8, 9, 10, 11, 12, 13, 14, 15]
                sage: B.directed_subset([7, 10], 'down')
                [0, 1, 2, 3, 4, 5, 6, 7, 8, 10]

            TESTS::

                sage: B = posets.BooleanLattice(3)
                sage: B.directed_subset([3, 1], 'banana')
                Traceback (most recent call last):
                ...
                ValueError: direction must be either 'up' or 'down'
            """
            if direction == 'up':
                return self.order_filter(elements)
            if direction == 'down':
                return self.order_ideal(elements)
            raise ValueError("direction must be either 'up' or 'down'")

        def principal_order_ideal(self, x):
            r"""
            Return the order ideal generated by an element ``x``.

            This is also called the lower set generated by this element.

            EXAMPLES::

                sage: B = posets.BooleanLattice(4)
                sage: B.principal_order_ideal(6)
                [0, 2, 4, 6]
            """
            return self.order_ideal([x])

        principal_lower_set = principal_order_ideal

        def principal_order_filter(self, x):
            r"""
            Return the order filter generated by an element ``x``.

            This is also called the upper set generated by this element.

            EXAMPLES::

                sage: B = posets.BooleanLattice(4)
                sage: B.principal_order_filter(2)
                [2, 3, 6, 7, 10, 11, 14, 15]
            """
            return self.order_filter([x])

        principal_upper_set = principal_order_filter

        def order_ideal_toggle(self, I, v):
            r"""
            Return the result of toggling the element ``v`` in the
            order ideal ``I``.

            If `v` is an element of a poset `P`, then toggling the
            element `v` is an automorphism of the set `J(P)` of all
            order ideals of `P`. It is defined as follows: If `I`
            is an order ideal of `P`, then the image of `I` under
            toggling the element `v` is

            - the set `I \cup \{ v \}`, if `v \not\in I` but
              every element of `P` smaller than `v` is in `I`;

            - the set `I \setminus \{ v \}`, if `v \in I` but
              no element of `P` greater than `v` is in `I`;

            - `I` otherwise.

            This image always is an order ideal of `P`.

            EXAMPLES::

                sage: P = Poset({1: [2,3], 2: [4], 3: []})
                sage: I = Set({1, 2})
                sage: I in P.order_ideals_lattice()                                     # needs sage.modules
                True
                sage: P.order_ideal_toggle(I, 1)
                {1, 2}
                sage: P.order_ideal_toggle(I, 2)
                {1}
                sage: P.order_ideal_toggle(I, 3)
                {1, 2, 3}
                sage: P.order_ideal_toggle(I, 4)
                {1, 2, 4}
                sage: P4 = Posets(4)
                sage: all(all(all(P.order_ideal_toggle(P.order_ideal_toggle(I, i), i) == I          # needs sage.modules
                ....:               for i in range(4))
                ....:          for I in P.order_ideals_lattice(facade=True))
                ....:     for P in P4)
                True
            """
            if v not in I:
                if all(u in I for u in self.lower_covers(v)):
                    from sage.sets.set import Set
                    return I.union(Set({v}))
            else:
                if all(u not in I for u in self.upper_covers(v)):
                    from sage.sets.set import Set
                    return I.difference(Set({v}))
            return I

        def order_ideal_toggles(self, I, vs):
            r"""
            Return the result of toggling the elements of the list (or
            iterable) ``vs`` (one by one, from left to right) in the order
            ideal ``I``.

            See :meth:`order_ideal_toggle` for a definition of toggling.

            EXAMPLES::

                sage: P = Poset({1: [2,3], 2: [4], 3: []})
                sage: I = Set({1, 2})
                sage: P.order_ideal_toggles(I, [1,2,3,4])
                {1, 3}
                sage: P.order_ideal_toggles(I, (1,2,3,4))
                {1, 3}
            """
            for v in vs:
                I = self.order_ideal_toggle(I, v)
            return I

        def is_order_ideal(self, o):
            """
            Return whether ``o`` is an order ideal of ``self``, assuming ``self``
            has no infinite descending path.

            INPUT:

            - ``o`` -- a list (or set, or tuple) containing some elements of ``self``

            EXAMPLES::

                sage: P = Poset((divisors(12), attrcall("divides")),
                ....:           facade=True, linear_extension=True)
                sage: sorted(P.list())
                [1, 2, 3, 4, 6, 12]
                sage: P.is_order_ideal([1, 3])
                True
                sage: P.is_order_ideal([])
                True
                sage: P.is_order_ideal({1, 3})
                True
                sage: P.is_order_ideal([1, 3, 4])
                False

            """
            return all((u in self and all(x in o for x in self.lower_covers(u))) for u in o)

        def is_order_filter(self, o):
            """
            Return whether ``o`` is an order filter of ``self``, assuming ``self``
            has no infinite ascending path.

            INPUT:

            - ``o`` -- a list (or set, or tuple) containing some elements of ``self``

            EXAMPLES::

                sage: P = Poset((divisors(12), attrcall("divides")),
                ....:           facade=True, linear_extension=True)
                sage: sorted(P.list())
                [1, 2, 3, 4, 6, 12]
                sage: P.is_order_filter([4, 12])
                True
                sage: P.is_order_filter([])
                True
                sage: P.is_order_filter({3, 4, 12})
                False
                sage: P.is_order_filter({3, 6, 12})
                True

            """
            return all((u in self and all(x in o for x in self.upper_covers(u))) for u in o)

        def is_chain_of_poset(self, o, ordered=False):
            """
            Return whether an iterable ``o`` is a chain of ``self``,
            including a check for ``o`` being ordered from smallest
            to largest element if the keyword ``ordered`` is set to
            ``True``.

            INPUT:

            - ``o`` -- an iterable (e. g., list, set, or tuple)
              containing some elements of ``self``

            - ``ordered`` -- a Boolean (default: ``False``) which
              decides whether the notion of a chain includes being
              ordered

            OUTPUT:

            If ``ordered`` is set to ``False``, the truth value of
            the following assertion is returned: The subset of ``self``
            formed by the elements of ``o`` is a chain in ``self``.

            If ``ordered`` is set to ``True``, the truth value of
            the following assertion is returned: Every element of the
            list ``o`` is (strictly!) smaller than its successor in
            ``self``. (This makes no sense if ``ordered`` is a set.)

            EXAMPLES::

                sage: P = Poset((divisors(12), attrcall("divides")),
                ....:           facade=True, linear_extension=True)
                sage: sorted(P.list())
                [1, 2, 3, 4, 6, 12]
                sage: P.is_chain_of_poset([1, 3])
                True
                sage: P.is_chain_of_poset([3, 1])
                True
                sage: P.is_chain_of_poset([1, 3], ordered=True)
                True
                sage: P.is_chain_of_poset([3, 1], ordered=True)
                False
                sage: P.is_chain_of_poset([])
                True
                sage: P.is_chain_of_poset([], ordered=True)
                True
                sage: P.is_chain_of_poset((2, 12, 6))
                True
                sage: P.is_chain_of_poset((2, 6, 12), ordered=True)
                True
                sage: P.is_chain_of_poset((2, 12, 6), ordered=True)
                False
                sage: P.is_chain_of_poset((2, 12, 6, 3))
                False
                sage: P.is_chain_of_poset((2, 3))
                False

                sage: Q = Poset({2: [3, 1], 3: [4], 1: [4]})
                sage: Q.is_chain_of_poset([1, 2], ordered=True)
                False
                sage: Q.is_chain_of_poset([1, 2])
                True
                sage: Q.is_chain_of_poset([2, 1], ordered=True)
                True
                sage: Q.is_chain_of_poset([2, 1, 1], ordered=True)
                False
                sage: Q.is_chain_of_poset([3])
                True
                sage: Q.is_chain_of_poset([4, 2, 3])
                True
                sage: Q.is_chain_of_poset([4, 2, 3], ordered=True)
                False
                sage: Q.is_chain_of_poset([2, 3, 4], ordered=True)
                True

            Examples with infinite posets::

                sage: from sage.categories.examples.posets import FiniteSetsOrderedByInclusion
                sage: R = FiniteSetsOrderedByInclusion()
                sage: R.is_chain_of_poset([R(set([3, 1, 2])),
                ....:                      R(set([1, 4])),
                ....:                      R(set([4, 5]))])
                False
                sage: R.is_chain_of_poset([R(set([3, 1, 2])),
                ....:                      R(set([1, 2])),
                ....:                      R(set([1]))], ordered=True)
                False
                sage: R.is_chain_of_poset([R(set([3, 1, 2])),
                ....:                      R(set([1, 2])), R(set([1]))])
                True

                sage: from sage.categories.examples.posets import PositiveIntegersOrderedByDivisibilityFacade
                sage: T = PositiveIntegersOrderedByDivisibilityFacade()
                sage: T.is_chain_of_poset((T(3), T(4), T(7)))
                False
                sage: T.is_chain_of_poset((T(3), T(6), T(3)))
                True
                sage: T.is_chain_of_poset((T(3), T(6), T(3)), ordered=True)
                False
                sage: T.is_chain_of_poset((T(3), T(3), T(6)))
                True
                sage: T.is_chain_of_poset((T(3), T(3), T(6)), ordered=True)
                False
                sage: T.is_chain_of_poset((T(3), T(6)), ordered=True)
                True
                sage: T.is_chain_of_poset((), ordered=True)
                True
                sage: T.is_chain_of_poset((T(3),), ordered=True)
                True
                sage: T.is_chain_of_poset((T(q) for q in divisors(27)))
                True
                sage: T.is_chain_of_poset((T(q) for q in divisors(18)))
                False
            """
            list_o = list(o)
            if ordered:
                return all(self.lt(a, b) for a, b in zip(list_o, list_o[1:]))
            else:
                for (i, x) in enumerate(list_o):
                    for y in list_o[:i]:
                        if (not self.le(x, y)) and (not self.gt(x, y)):
                            return False
                return True

        def is_antichain_of_poset(self, o):
            """
            Return whether an iterable ``o`` is an antichain of
            ``self``.

            INPUT:

            - ``o`` -- an iterable (e. g., list, set, or tuple)
              containing some elements of ``self``

            OUTPUT:

            ``True`` if the subset of ``self`` consisting of the entries
            of ``o`` is an antichain of ``self``, and ``False`` otherwise.

            EXAMPLES::

                sage: P = Poset((divisors(12), attrcall("divides")),
                ....:           facade=True, linear_extension=True)
                sage: sorted(P.list())
                [1, 2, 3, 4, 6, 12]
                sage: P.is_antichain_of_poset([1, 3])
                False
                sage: P.is_antichain_of_poset([3, 1])
                False
                sage: P.is_antichain_of_poset([1, 1, 3])
                False
                sage: P.is_antichain_of_poset([])
                True
                sage: P.is_antichain_of_poset([1])
                True
                sage: P.is_antichain_of_poset([1, 1])
                True
                sage: P.is_antichain_of_poset([3, 4])
                True
                sage: P.is_antichain_of_poset([3, 4, 12])
                False
                sage: P.is_antichain_of_poset([6, 4])
                True
                sage: P.is_antichain_of_poset(i for i in divisors(12)
                ....:                         if (2 < i and i < 6))
                True
                sage: P.is_antichain_of_poset(i for i in divisors(12)
                ....:                         if (2 <= i and i < 6))
                False

                sage: Q = Poset({2: [3, 1], 3: [4], 1: [4]})
                sage: Q.is_antichain_of_poset((1, 2))
                False
                sage: Q.is_antichain_of_poset((2, 4))
                False
                sage: Q.is_antichain_of_poset((4, 2))
                False
                sage: Q.is_antichain_of_poset((2, 2))
                True
                sage: Q.is_antichain_of_poset((3, 4))
                False
                sage: Q.is_antichain_of_poset((3, 1))
                True
                sage: Q.is_antichain_of_poset((1, ))
                True
                sage: Q.is_antichain_of_poset(())
                True

            An infinite poset::

                sage: from sage.categories.examples.posets import FiniteSetsOrderedByInclusion
                sage: R = FiniteSetsOrderedByInclusion()
                sage: R.is_antichain_of_poset([R(set([3, 1, 2])),
                ....:                          R(set([1, 4])), R(set([4, 5]))])
                True
                sage: R.is_antichain_of_poset([R(set([3, 1, 2, 4])),
                ....:                          R(set([1, 4])), R(set([4, 5]))])
                False
            """
            return all(not self.lt(x,y) for x in o for y in o)

        CartesianProduct = LazyImport(
            'sage.combinat.posets.cartesian_product', 'CartesianProductPoset')

    class ElementMethods:
        pass
        # TODO: implement x<y, x<=y, x>y, x>=y appropriately once #10130 is resolved
        #
        # def __le__(self, other):
        #     r"""
        #     Return whether ``self`` is smaller or equal to ``other``
        #     in the poset.
        #
        #     EXAMPLES::
        #
        #         sage: P = Posets().example(); P
        #         An example of poset: sets ordered by inclusion
        #         sage: x = P(Set([1,3])); y = P(Set([1,2,3]))
        #         sage: x.__le__(y)
        #         sage: x <= y
        #     """
        #     return self.parent().le(self, other)



# sage_setup: distribution = sagemath-categories
r"""
Examples of semigroups in cython
"""

from sage.structure.parent cimport Parent
from sage.structure.element cimport Element
from sage.categories.category import Category
from sage.categories.semigroups import Semigroups
from sage.categories.examples.semigroups import LeftZeroSemigroup as LeftZeroSemigroupPython
from cpython.object cimport PyObject_RichCompare


class IdempotentSemigroups(Category):
    def super_categories(self):
        """
        EXAMPLES::

            sage: from sage.categories.examples.semigroups_cython import IdempotentSemigroups
            sage: IdempotentSemigroups().super_categories()
            [Category of semigroups]
        """
        return [Semigroups()]

    class ElementMethods:
        def _pow_int(self, i):
            """
            EXAMPLES::

                sage: from sage.categories.examples.semigroups_cython import LeftZeroSemigroup
                sage: S = LeftZeroSemigroup()
                sage: S(2)._pow_int(3)
                2
            """
            assert i > 0
            return self

        def is_idempotent(self):
            """
            EXAMPLES::

                sage: from sage.categories.examples.semigroups_cython import LeftZeroSemigroup
                sage: S = LeftZeroSemigroup()
                sage: S(2).is_idempotent()
                True
            """
            return True


cdef class LeftZeroSemigroupElement(Element):
    cdef object _value

    def __init__(self, parent, value):
        """
        EXAMPLES::

            sage: from sage.categories.examples.semigroups_cython import LeftZeroSemigroup
            sage: S = LeftZeroSemigroup()
            sage: x = S(3)
            sage: TestSuite(x).run()
        """
        Element.__init__(self, parent=parent)
        self._value = value

    def _repr_(self):
        """
        EXAMPLES::

            sage: from sage.categories.examples.semigroups_cython import LeftZeroSemigroup
            sage: S = LeftZeroSemigroup()
            sage: S(3)                    # indirect doctest
            3
        """
        return repr(self._value)

    def __reduce__(self):
        """
        EXAMPLES::

            sage: from sage.categories.examples.semigroups_cython import LeftZeroSemigroup
            sage: S = LeftZeroSemigroup()
            sage: x = S(3)
            sage: x.__reduce__()
            (<class 'sage.categories.examples.semigroups_cython.LeftZeroSemigroupElement'>,
             (An example of a semigroup: the left zero semigroup, 3))
        """
        return LeftZeroSemigroupElement, (self._parent, self._value)

    cpdef _richcmp_(self, other, int op):
        """
        EXAMPLES::

            sage: from sage.categories.examples.semigroups_cython import LeftZeroSemigroup
            sage: S = LeftZeroSemigroup()
            sage: S(3) == S(3)
            True
            sage: S(3) == S(2)
            False
        """
        left = (<LeftZeroSemigroupElement>self)._value
        right = (<LeftZeroSemigroupElement>other)._value
        return PyObject_RichCompare(left, right, op)

    cpdef _mul_(self, other):
        """
        EXAMPLES::

            sage: from sage.categories.examples.semigroups_cython import LeftZeroSemigroup
            sage: S = LeftZeroSemigroup()
            sage: S(2) * S(3)
            2
            sage: S(2)._mul_(S(3))
            2
        """
        return self.parent().product(self, other)

    def __pow__(self, i, dummy):
        """
        EXAMPLES::

            sage: from sage.categories.examples.semigroups_cython import LeftZeroSemigroup
            sage: S = LeftZeroSemigroup()
            sage: S(2)^3
            2
        """
        return self._pow_int(i)


class LeftZeroSemigroup(LeftZeroSemigroupPython):
    r"""
    An example of semigroup

    This class illustrates a minimal implementation of a semi-group
    where the element class is an extension type, and still gets code
    from the category. The category itself must be a Python class
    though.

    This is purely a proof of concept. The code obviously needs refactorisation!

    Comments:

    - one cannot play ugly class surgery tricks (as with _mul_parent).
      available operations should really be declared to the coercion model!

    EXAMPLES::

        sage: from sage.categories.examples.semigroups_cython import LeftZeroSemigroup
        sage: S = LeftZeroSemigroup(); S
        An example of a semigroup: the left zero semigroup

    This is the semigroup which contains all sort of objects::

        sage: S.some_elements()
        [3, 42, 'a', 3.4, 'raton laveur']

    with product rule given by `a \times b = a` for all `a,b`. ::

        sage: S('hello') * S('world')
        'hello'

        sage: S(3)*S(1)*S(2)
        3

        sage: S(3)^12312321312321
        3

        sage: TestSuite(S).run(verbose = True)
        running ._test_an_element() . . . pass
        running ._test_associativity() . . . pass
        running ._test_cardinality() . . . pass
        running ._test_category() . . . pass
        running ._test_construction() . . . pass
        running ._test_elements() . . .
          Running the test suite of self.an_element()
          running ._test_category() . . . pass
          running ._test_eq() . . . pass
          running ._test_new() . . . pass
          running ._test_not_implemented_methods() . . . pass
          running ._test_pickling() . . . pass
          pass
        running ._test_elements_eq_reflexive() . . . pass
        running ._test_elements_eq_symmetric() . . . pass
        running ._test_elements_eq_transitive() . . . pass
        running ._test_elements_neq() . . . pass
        running ._test_eq() . . . pass
        running ._test_new() . . . pass
        running ._test_not_implemented_methods() . . . pass
        running ._test_pickling() . . . pass
        running ._test_some_elements() . . . pass

    That's really the only method which is obtained from the category ... ::

        sage: S(42).is_idempotent
        <bound method IdempotentSemigroups.ElementMethods.is_idempotent of 42>
        sage: S(42).is_idempotent()
        True

        sage: S(42)._pow_int
        <bound method IdempotentSemigroups.ElementMethods._pow_int of 42>
        sage: S(42)^10
        42

        sage: S(42).is_idempotent
        <bound method IdempotentSemigroups.ElementMethods.is_idempotent of 42>
        sage: S(42).is_idempotent()
        True
    """

    def __init__(self):
        """
        TESTS::

            sage: from sage.categories.examples.semigroups_cython import LeftZeroSemigroup
            sage: S = LeftZeroSemigroup()
            sage: S.category()
            Category of idempotent semigroups
            sage: TestSuite(S).run()
        """
        Parent.__init__(self, category=IdempotentSemigroups())

    Element = LeftZeroSemigroupElement



# sage_setup: distribution = sagemath-categories
# sage.doctest: needs sage.combinat
r"""
Examples of algebras with basis
"""
#*****************************************************************************
#  Copyright (C) 2008-2009 Nicolas M. Thiery <nthiery at users.sf.net>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#                  http://www.gnu.org/licenses/
#*****************************************************************************

from sage.misc.cachefunc import cached_method
from sage.sets.family import Family
from sage.categories.algebras_with_basis import AlgebrasWithBasis
from sage.combinat.free_module import CombinatorialFreeModule
from sage.combinat.words.words import Words


class FreeAlgebra(CombinatorialFreeModule):
    r"""
    An example of an algebra with basis: the free algebra

    This class illustrates a minimal implementation of an algebra with basis.
    """

    def __init__(self, R, alphabet=("a", "b", "c")):
        """
        EXAMPLES::

            sage: A = AlgebrasWithBasis(QQ).example(); A                                # needs sage.modules
            An example of an algebra with basis: the free algebra on the generators ('a', 'b', 'c') over Rational Field
            sage: TestSuite(A).run()                                                    # needs sage.modules

        """
        self._alphabet = alphabet
        CombinatorialFreeModule.__init__(self, R,
                                         Words(alphabet, infinite=False),
                                         category=AlgebrasWithBasis(R))

    def _repr_(self):
        """
        EXAMPLES::

            sage: AlgebrasWithBasis(QQ).example()  # indirect doctest                   # needs sage.modules
            An example of an algebra with basis: the free algebra on the generators ('a', 'b', 'c') over Rational Field
        """
        return "An example of an algebra with basis: the free algebra on the generators %s over %s" % (self._alphabet, self.base_ring())

    @cached_method
    def one_basis(self):
        """
        Returns the empty word, which index the one of this algebra,
        as per :meth:`AlgebrasWithBasis.ParentMethods.one_basis`.

        EXAMPLES::r

            sage: A = AlgebrasWithBasis(QQ).example()                                   # needs sage.modules
            sage: A.one_basis()                                                         # needs sage.modules
            word:
            sage: A.one()                                                               # needs sage.modules
            B[word: ]
        """
        return self.basis().keys()([])

    def product_on_basis(self, w1, w2):
        r"""
        Product of basis elements, as per
        :meth:`AlgebrasWithBasis.ParentMethods.product_on_basis`.

        EXAMPLES::

            sage: # needs sage.modules
            sage: A = AlgebrasWithBasis(QQ).example()
            sage: Words = A.basis().keys()
            sage: A.product_on_basis(Words("acb"), Words("cba"))
            B[word: acbcba]
            sage: (a,b,c) = A.algebra_generators()
            sage: a * (1-b)^2 * c
            B[word: abbc] - 2*B[word: abc] + B[word: ac]
        """
        return self.basis()[w1 + w2]

    @cached_method
    def algebra_generators(self):
        r"""
        Return the generators of this algebra, as per :meth:`~.magmatic_algebras.MagmaticAlgebras.ParentMethods.algebra_generators`.

        EXAMPLES::

            sage: A = AlgebrasWithBasis(QQ).example(); A                                # needs sage.modules
            An example of an algebra with basis: the free algebra on the generators ('a', 'b', 'c') over Rational Field
            sage: A.algebra_generators()                                                # needs sage.modules
            Family (B[word: a], B[word: b], B[word: c])
        """
        Words = self.basis().keys()
        return Family( [self.monomial(Words(a)) for a in self._alphabet] )
        # FIXME: use this once the keys argument of FiniteFamily will be honoured
        # for the specifying the order of the elements in the family
        #return Family(self._alphabet, lambda a: self.term(self.basis().keys()(a)))


Example = FreeAlgebra



# sage_setup: distribution = sagemath-categories
r"""
Graded modules with basis
"""
#*****************************************************************************
#  Copyright (C) 2008      Teresa Gomez-Diaz (CNRS) <Teresa.Gomez-Diaz@univ-mlv.fr>
#                2008-2011 Nicolas M. Thiery <nthiery at users.sf.net>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#                  http://www.gnu.org/licenses/
#******************************************************************************

from sage.categories.graded_modules import GradedModulesCategory
from sage.categories.quotients import QuotientsCategory


class GradedModulesWithBasis(GradedModulesCategory):
    """
    The category of graded modules with a distinguished basis.

    EXAMPLES::

        sage: C = GradedModulesWithBasis(ZZ); C
        Category of graded modules with basis over Integer Ring
        sage: sorted(C.super_categories(), key=str)
        [Category of filtered modules with basis over Integer Ring,
         Category of graded modules over Integer Ring]
        sage: C is ModulesWithBasis(ZZ).Graded()
        True

    TESTS::

        sage: TestSuite(C).run()
    """
    class ParentMethods:
        def degree_negation(self, element):
            r"""
            Return the image of ``element`` under the degree negation
            automorphism of the graded module ``self``.

            The degree negation is the module automorphism which scales
            every homogeneous element of degree `k` by `(-1)^k` (for all
            `k`). This assumes that the module ``self`` is `\ZZ`-graded.

            INPUT:

            - ``element`` -- element of the module ``self``

            EXAMPLES::

                sage: E.<a,b> = ExteriorAlgebra(QQ)                                     # needs sage.modules
                sage: E.degree_negation((1 + a) * (1 + b))                              # needs sage.modules
                a*b - a - b + 1
                sage: E.degree_negation(E.zero())                                       # needs sage.modules
                0

                sage: P = GradedModulesWithBasis(ZZ).example(); P                       # needs sage.combinat sage.modules
                An example of a graded module with basis:
                 the free module on partitions over Integer Ring
                sage: pbp = lambda x: P.basis()[Partition(list(x))]
                sage: p = pbp([3,1]) - 2 * pbp([2]) + 4 * pbp([1])                      # needs sage.combinat sage.modules
                sage: P.degree_negation(p)                                              # needs sage.combinat sage.modules
                -4*P[1] - 2*P[2] + P[3, 1]
            """
            base_one = self.base_ring().one()
            base_minusone = - base_one
            diag = lambda x: (base_one if self.degree_on_basis(x) % 2 == 0
                              else base_minusone)
            return self.sum_of_terms([(key, diag(key) * value)
                                      for key, value in
                                      element.monomial_coefficients(copy=False).items()])

        def submodule(self, gens, check=True, already_echelonized=False,
                      unitriangular=False, support_order=None, category=None,
                      *args, **opts):
            r"""
            Return the submodule spanned by a finite set of elements.

            INPUT:

            - ``gens`` -- a list or family of elements of ``self``
            - ``check`` -- (default: ``True``) whether to verify that the
               elements of ``gens`` are in ``self``
            - ``already_echelonized`` -- (default: ``False``) whether
               the elements of ``gens`` are already in (not necessarily
               reduced) echelon form
            - ``unitriangular`` -- (default: ``False``) whether
              the lift morphism is unitriangular
            - ``support_order`` -- (optional) either something that can
              be converted into a tuple or a key function
            - ``category`` -- (optional) the category of the submodule

            If ``already_echelonized`` is ``False``, then the
            generators are put in reduced echelon form using
            :meth:`echelonize`, and reindexed by `0,1,...`.

            .. WARNING::

                At this point, this method only works for finite
                dimensional submodules and if matrices can be
                echelonized over the base ring.

            If in addition ``unitriangular`` is ``True``, then
            the generators are made such that the coefficients of
            the pivots are 1, so that lifting map is unitriangular.

            The basis of the submodule uses the same index set as the
            generators, and the lifting map sends `y_i` to `gens[i]`.

            .. SEEALSO::

                 - :meth:`ModulesWithBasis.FiniteDimensional.ParentMethods.quotient_module`
                 - :class:`sage.modules.with_basis.subquotient.SubmoduleWithBasis`

            EXAMPLES:

            A graded submodule of a graded module generated by homogeneous
            elements is naturally graded::

                sage: # needs sage.combinat sage.modules
                sage: E.<x,y,z> = ExteriorAlgebra(QQ)
                sage: S = E.submodule([x + y, x*y - y*z])
                sage: S.category()
                Join of
                 Category of graded vector spaces with basis over Rational Field and
                 Category of subobjects of filtered modules with basis over Rational Field and
                 Category of finite dimensional filtered modules with basis over Rational Field
                sage: S.basis()[0].degree()
                1
                sage: S.basis()[1].degree()
                2

            We check on the echelonized basis::

                sage: Sp = E.submodule([1, x + y + 5, x*y - y*z + x + y - 2])           # needs sage.combinat sage.modules
                sage: Sp.category()                                                     # needs sage.combinat sage.modules
                Join of
                 Category of graded vector spaces with basis over Rational Field and
                 Category of subobjects of filtered modules with basis over Rational Field and
                 Category of finite dimensional filtered modules with basis over Rational Field

            If it is generated by inhomogeneous elements, then it is
            filtered by default::

                sage: F = E.submodule([x + y*z, x*z + y*x])                             # needs sage.combinat sage.modules
                sage: F.category()                                                      # needs sage.combinat sage.modules
                Join of
                 Category of subobjects of filtered modules with basis over Rational Field and
                 Category of finite dimensional filtered modules with basis over Rational Field and
                 Category of filtered vector spaces with basis over Rational Field

            If ``category`` is specified, then it does not give any extra
            structure to the submodule (we can think of this as applying
            the forgetful functor)::

                sage: # needs sage.combinat sage.modules
                sage: SM = E.submodule([x + y, x*y - y*z],
                ....:                  category=ModulesWithBasis(QQ))
                sage: SM.category()
                Join of
                 Category of finite dimensional vector spaces with basis over Rational Field and
                 Category of subobjects of sets
                sage: FM = E.submodule([x + 1, x*y - x*y*z],
                ....:                  category=ModulesWithBasis(QQ))
                sage: FM.category()
                Join of
                 Category of finite dimensional vector spaces with basis over Rational Field and
                 Category of subobjects of sets

            If we have specified that this is a graded submodule of a graded
            module, then the echelonized elements must be homogeneous::

                sage: Cat = ModulesWithBasis(QQ).Graded().Subobjects()
                sage: E.submodule([x + y, x*y - 1], category=Cat)                       # needs sage.combinat sage.modules
                Traceback (most recent call last):
                ...
                ValueError: all of the generators must be homogeneous
                sage: E.submodule([x + y, x*y - x - y], category=Cat)                   # needs sage.combinat sage.modules
                Free module generated by {0, 1} over Rational Field
            """
            # Make sure gens consists of elements of ``self``
            from sage.sets.family import Family, AbstractFamily
            if isinstance(gens, AbstractFamily):
                gens = gens.map(self)
            elif isinstance(gens, dict):
                gens = Family(gens.keys(), gens.__getitem__)
            else:
                gens = [self(y) for y in gens]
            support_order = self._compute_support_order(gens, support_order)
            if not already_echelonized:
                gens = self.echelon_form(gens, unitriangular, order=support_order)

            GMod = GradedModulesWithBasis(self.category().base_ring())
            if category is None:
                if all(g.is_homogeneous() for g in gens):
                    category = GMod.Subobjects()
            elif (category.is_subcategory(GMod.Subobjects())
                  and not all(g.is_homogeneous() for g in gens)):
                raise ValueError("all of the generators must be homogeneous")

            from sage.modules.with_basis.subquotient import SubmoduleWithBasis
            return SubmoduleWithBasis(gens, ambient=self,
                                      support_order=support_order,
                                      unitriangular=unitriangular,
                                      category=category, *args, **opts)

        def quotient_module(self, submodule, check=True, already_echelonized=False, category=None):
            r"""
            Construct the quotient module ``self`` / ``submodule``.

            INPUT:

            - ``submodule`` -- a submodule with basis of ``self``, or
              something that can be turned into one via
              ``self.submodule(submodule)``
            - ``check``, ``already_echelonized`` -- passed down to
              :meth:`ModulesWithBasis.ParentMethods.submodule`
            - ``category`` -- (optional) the category of the quotient module

            .. WARNING::

                At this point, this only supports quotients by free
                submodules admitting a basis in unitriangular echelon
                form. In this case, the quotient is also a free
                module, with a basis consisting of the retract of a
                subset of the basis of ``self``.

            EXAMPLES::

                sage: # needs sage.combinat sage.modules
                sage: E.<x,y,z> = ExteriorAlgebra(QQ)
                sage: S = E.submodule([x + y, x*y - y*z, y])
                sage: Q = E.quotient_module(S)
                sage: Q.category()
                Join of
                 Category of quotients of graded modules with basis over Rational Field and
                 Category of graded vector spaces with basis over Rational Field and
                 Category of finite dimensional filtered modules with basis over Rational Field

            .. SEEALSO::

                 - :meth:`Modules.WithBasis.ParentMethods.submodule`
                 - :meth:`Rings.ParentMethods.quotient`
                 - :class:`sage.modules.with_basis.subquotient.QuotientModuleWithBasis`
            """
            from sage.modules.with_basis.subquotient import SubmoduleWithBasis, QuotientModuleWithBasis
            if not isinstance(submodule, SubmoduleWithBasis):
                submodule = self.submodule(submodule, check=check,
                                           unitriangular=True,
                                           already_echelonized=already_echelonized)

            GMod = GradedModulesWithBasis(self.category().base_ring())
            if category is None:
                if all(g.is_homogeneous() for g in submodule.basis()):
                    category = GMod.Quotients()
            elif (category.is_subcategory(GMod.Quotients())
                  and not all(g.is_homogeneous() for g in submodule.basis())):
                raise ValueError("all of the basis elements must be homogeneous")

            return QuotientModuleWithBasis(submodule, category=category)

    class ElementMethods:
        def degree_negation(self):
            r"""
            Return the image of ``self`` under the degree negation
            automorphism of the graded module to which ``self`` belongs.

            The degree negation is the module automorphism which scales
            every homogeneous element of degree `k` by `(-1)^k` (for all
            `k`). This assumes that the module to which ``self`` belongs
            (that is, the module ``self.parent()``) is `\ZZ`-graded.

            EXAMPLES::

                sage: E.<a,b> = ExteriorAlgebra(QQ)                                     # needs sage.modules
                sage: ((1 + a) * (1 + b)).degree_negation()                             # needs sage.modules
                a*b - a - b + 1
                sage: E.zero().degree_negation()                                        # needs sage.modules
                0

                sage: P = GradedModulesWithBasis(ZZ).example(); P                       # needs sage.combinat sage.modules
                An example of a graded module with basis:
                 the free module on partitions over Integer Ring
                sage: pbp = lambda x: P.basis()[Partition(list(x))]
                sage: p = pbp([3,1]) - 2 * pbp([2]) + 4 * pbp([1])                      # needs sage.combinat sage.modules
                sage: p.degree_negation()                                               # needs sage.combinat sage.modules
                -4*P[1] - 2*P[2] + P[3, 1]
            """
            return self.parent().degree_negation(self)

    class Quotients(QuotientsCategory):
        class ParentMethods:
            def degree_on_basis(self, m):
                r"""
                Return the degree of the basis element indexed by ``m``
                in ``self``.

                EXAMPLES::

                    sage: # needs sage.combinat sage.modules
                    sage: E.<x,y,z> = ExteriorAlgebra(QQ)
                    sage: S = E.submodule([x + y, x*y - y*z, y])
                    sage: Q = E.quotient_module(S)
                    sage: B = Q.basis()
                    sage: [B[i].lift() for i in Q.indices()]
                    [1, z, x*z, y*z, x*y*z]
                    sage: [Q.degree_on_basis(i) for i in Q.indices()]
                    [0, 1, 2, 2, 3]
                """
                return self.basis()[m].lift().degree()

        class ElementMethods:
            def degree(self):
                r"""
                Return the degree of ``self``.

                EXAMPLES::

                    sage: # needs sage.combinat sage.modules
                    sage: E.<x,y,z> = ExteriorAlgebra(QQ)
                    sage: S = E.submodule([x + y, x*y - y*z, y])
                    sage: Q = E.quotient_module(S)
                    sage: B = Q.basis()
                    sage: [B[i].lift() for i in Q.indices()]
                    [1, z, x*z, y*z, x*y*z]
                    sage: [B[i].degree() for i in Q.indices()]
                    [0, 1, 2, 2, 3]
                """
                return self.lift().degree()



# sage_setup: distribution = sagemath-categories
r"""
Super algebras with basis
"""
# ****************************************************************************
#  Copyright (C) 2015,2019 Travis Scrimshaw <tcscrims at gmail.com>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#                  https://www.gnu.org/licenses/
# *****************************************************************************

from sage.categories.super_modules import SuperModulesCategory
from sage.categories.signed_tensor import SignedTensorProductsCategory
from sage.misc.cachefunc import cached_method


class SuperAlgebrasWithBasis(SuperModulesCategory):
    """
    The category of super algebras with a distinguished basis

    EXAMPLES::

        sage: C = Algebras(ZZ).WithBasis().Super(); C
        Category of super algebras with basis over Integer Ring

    TESTS::

        sage: TestSuite(C).run()
    """
    def extra_super_categories(self):
        """
        EXAMPLES::

            sage: C = Algebras(ZZ).WithBasis().Super()
            sage: sorted(C.super_categories(), key=str) # indirect doctest
            [Category of graded algebras with basis over Integer Ring,
             Category of super algebras over Integer Ring,
             Category of super modules with basis over Integer Ring]
        """
        return [self.base_category().Graded()]

    class ParentMethods:
        def graded_algebra(self):
            r"""
            Return the associated graded module to ``self``.

            See :class:`~sage.algebras.associated_graded.AssociatedGradedAlgebra`
            for the definition and the properties of this.

            .. SEEALSO::

                :meth:`~sage.categories.filtered_modules_with_basis.ParentMethods.graded_algebra`

            EXAMPLES::

                sage: W.<x,y> = algebras.DifferentialWeyl(QQ)                           # needs sage.modules
                sage: W.graded_algebra()                                                # needs sage.combinat sage.modules
                Graded Algebra of Differential Weyl algebra of
                 polynomials in x, y over Rational Field
            """
            from sage.algebras.associated_graded import AssociatedGradedAlgebra
            return AssociatedGradedAlgebra(self)

    class ElementMethods:
        def supercommutator(self, x):
            r"""
            Return the supercommutator of ``self`` and ``x``.

            Let `A` be a superalgebra. The *supercommutator* of homogeneous
            elements `x, y \in A` is defined by

            .. MATH::

                [x, y\} = x y - (-1)^{|x| |y|} y x

            and extended to all elements by linearity.

            EXAMPLES::

                sage: # needs sage.modules
                sage: Q = QuadraticForm(ZZ, 3, [1,2,3,4,5,6])
                sage: Cl.<x,y,z> = CliffordAlgebra(Q)
                sage: a = x*y - z
                sage: b = x - y + y*z
                sage: a.supercommutator(b)
                -5*x*y + 8*x*z - 2*y*z - 6*x + 12*y - 5*z
                sage: a.supercommutator(Cl.one())
                0
                sage: Cl.one().supercommutator(a)
                0
                sage: Cl.zero().supercommutator(a)
                0
                sage: a.supercommutator(Cl.zero())
                0

                sage: # needs sage.modules
                sage: Q = QuadraticForm(ZZ, 2, [-1,1,-3])
                sage: Cl.<x,y> = CliffordAlgebra(Q)
                sage: [a.supercommutator(b) for a in Cl.basis() for b in Cl.basis()]
                [0, 0, 0, 0, 0, -2, 1, -x - 2*y, 0, 1,
                 -6, 6*x + y, 0, x + 2*y, -6*x - y, 0]
                sage: [a*b-b*a for a in Cl.basis() for b in Cl.basis()]
                [0, 0, 0, 0, 0, 0, 2*x*y - 1, -x - 2*y, 0,
                 -2*x*y + 1, 0, 6*x + y, 0, x + 2*y, -6*x - y, 0]

            Exterior algebras inherit from Clifford algebras, so
            supercommutators work as well. We verify the exterior algebra
            is supercommutative::

                sage: E.<x,y,z,w> = ExteriorAlgebra(QQ)                                 # needs sage.modules
                sage: all(b1.supercommutator(b2) == 0                                   # needs sage.modules
                ....:     for b1 in E.basis() for b2 in E.basis())
                True
            """
            P = self.parent()
            ret = P.zero()
            for ms, cs in self:
                term_s = P.term(ms, cs)
                sign_s = (-1)**P.degree_on_basis(ms)
                for mx, cx in x:
                    ret += term_s * P.term(mx, cx)
                    s = sign_s**P.degree_on_basis(mx)
                    ret -= s * P.term(mx, cx) * term_s
            return ret

    class SignedTensorProducts(SignedTensorProductsCategory):
        """
        The category of super algebras with basis constructed by tensor
        product of super algebras with basis.
        """
        @cached_method
        def extra_super_categories(self):
            """
            EXAMPLES::

                sage: Algebras(QQ).Super().SignedTensorProducts().extra_super_categories()
                [Category of super algebras over Rational Field]
                sage: Algebras(QQ).Super().SignedTensorProducts().super_categories()
                [Category of signed tensor products of graded algebras over Rational Field,
                 Category of super algebras over Rational Field]

            Meaning: a signed tensor product of super algebras is a super algebra
            """
            return [self.base_category()]



# sage_setup: distribution = sagemath-categories
r"""
Additive semigroups
"""
# ****************************************************************************
#  Copyright (C) 2013 Nicolas M. Thiery <nthiery at users.sf.net>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#                  https://www.gnu.org/licenses/
# *****************************************************************************

from sage.misc.cachefunc import cached_method
from sage.misc.lazy_import import LazyImport
from sage.categories.category_with_axiom import CategoryWithAxiom_singleton
from sage.categories.cartesian_product import CartesianProductsCategory
from sage.categories.algebra_functor import AlgebrasCategory
from sage.categories.homsets import HomsetsCategory
from sage.categories.additive_magmas import AdditiveMagmas


class AdditiveSemigroups(CategoryWithAxiom_singleton):
    """
    The category of additive semigroups.

    An *additive semigroup* is an associative :class:`additive magma
    <AdditiveMagmas>`, that is a set endowed with an operation `+`
    which is associative.

    EXAMPLES::

        sage: from sage.categories.additive_semigroups import AdditiveSemigroups
        sage: C = AdditiveSemigroups(); C
        Category of additive semigroups
        sage: C.super_categories()
        [Category of additive magmas]
        sage: C.all_super_categories()
        [Category of additive semigroups,
         Category of additive magmas,
         Category of sets,
         Category of sets with partial maps,
         Category of objects]

        sage: C.axioms()
        frozenset({'AdditiveAssociative'})
        sage: C is AdditiveMagmas().AdditiveAssociative()
        True

    TESTS::

        sage: TestSuite(C).run()
    """
    _base_category_class_and_axiom = (AdditiveMagmas, "AdditiveAssociative")

    AdditiveCommutative = LazyImport('sage.categories.commutative_additive_semigroups', 'CommutativeAdditiveSemigroups', at_startup=True)
    AdditiveUnital = LazyImport('sage.categories.additive_monoids', 'AdditiveMonoids', at_startup=True)

    class ParentMethods:
        def _test_additive_associativity(self, **options):
            r"""
            Test associativity for (not necessarily all) elements of this
            additive semigroup.

            INPUT:

            - ``options`` -- any keyword arguments accepted by :meth:`_tester`

            EXAMPLES:

            By default, this method tests only the elements returned by
            ``self.some_elements()``::

                sage: S = CommutativeAdditiveSemigroups().example()
                sage: S._test_additive_associativity()

            However, the elements tested can be customized with the
            ``elements`` keyword argument::

                sage: (a,b,c,d) = S.additive_semigroup_generators()
                sage: S._test_additive_associativity(elements = (a, b+c, d))

            See the documentation for :class:`TestSuite` for more information.
            """
            tester = self._tester(**options)
            S = tester.some_elements()
            from sage.misc.misc import some_tuples
            for x, y, z in some_tuples(S, 3, tester._max_runs):
                tester.assertEqual((x + y) + z, x + (y + z))

    class Homsets(HomsetsCategory):

        def extra_super_categories(self):
            r"""
            Implement the fact that a homset between two semigroups is a
            semigroup.

            EXAMPLES::

                sage: from sage.categories.additive_semigroups import AdditiveSemigroups
                sage: AdditiveSemigroups().Homsets().extra_super_categories()
                [Category of additive semigroups]
                sage: AdditiveSemigroups().Homsets().super_categories()
                [Category of homsets of additive magmas, Category of additive semigroups]
            """
            return [AdditiveSemigroups()]

    class CartesianProducts(CartesianProductsCategory):

        def extra_super_categories(self):
            """
            Implement the fact that a Cartesian product of additive semigroups
            is an additive semigroup.

            EXAMPLES::

                sage: from sage.categories.additive_semigroups import AdditiveSemigroups
                sage: C = AdditiveSemigroups().CartesianProducts()
                sage: C.extra_super_categories()
                [Category of additive semigroups]
                sage: C.axioms()
                frozenset({'AdditiveAssociative'})
            """
            return [AdditiveSemigroups()]

    class Algebras(AlgebrasCategory):

        def extra_super_categories(self):
            """
            EXAMPLES::

                sage: from sage.categories.additive_semigroups import AdditiveSemigroups
                sage: AdditiveSemigroups().Algebras(QQ).extra_super_categories()
                [Category of semigroups]
                sage: CommutativeAdditiveSemigroups().Algebras(QQ).super_categories()
                [Category of additive semigroup algebras over Rational Field,
                 Category of additive commutative additive magma algebras over Rational Field]
            """
            from sage.categories.semigroups import Semigroups
            return [Semigroups()]

        class ParentMethods:

            @cached_method
            def algebra_generators(self):
                r"""
                Return the generators of this algebra, as per
                :meth:`MagmaticAlgebras.ParentMethods.algebra_generators()
                <.magmatic_algebras.MagmaticAlgebras.ParentMethods.algebra_generators>`.

                They correspond to the generators of the additive semigroup.

                EXAMPLES::

                    sage: S = CommutativeAdditiveSemigroups().example(); S
                    An example of a commutative semigroup:
                     the free commutative semigroup generated by ('a', 'b', 'c', 'd')
                    sage: A = S.algebra(QQ)                                             # needs sage.modules
                    sage: A.algebra_generators()                                        # needs sage.modules
                    Family (B[a], B[b], B[c], B[d])
                """
                return self.basis().keys().additive_semigroup_generators().map(self.monomial)

            def product_on_basis(self, g1, g2):
                r"""
                Product, on basis elements, as per
                :meth:`MagmaticAlgebras.WithBasis.ParentMethods.product_on_basis()
                <sage.categories.magmatic_algebras.MagmaticAlgebras.WithBasis.ParentMethods.product_on_basis>`.

                The product of two basis elements is induced by the
                addition of the corresponding elements of the group.

                EXAMPLES::

                    sage: S = CommutativeAdditiveSemigroups().example(); S
                    An example of a commutative semigroup:
                     the free commutative semigroup generated by ('a', 'b', 'c', 'd')
                    sage: A = S.algebra(QQ)                                             # needs sage.modules
                    sage: a, b, c, d = A.algebra_generators()                           # needs sage.modules
                    sage: b * d * c                                                     # needs sage.modules
                    B[b + c + d]
                """
                return self.monomial(g1 + g2)



# sage_setup: distribution = sagemath-categories
# sage.doctest: needs sage.modules
r"""
Examples of graded connected Hopf algebras with basis
"""
# ****************************************************************************
#  Copyright (C) 2015 Jean-Baptiste Priez <jbp@kerios.fr>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#                  https://www.gnu.org/licenses/
# ****************************************************************************

from sage.categories.graded_hopf_algebras_with_basis import GradedHopfAlgebrasWithBasis
from sage.combinat.free_module import CombinatorialFreeModule
from sage.arith.misc import binomial
from sage.misc.cachefunc import cached_method
from sage.sets.non_negative_integers import NonNegativeIntegers


class GradedConnectedCombinatorialHopfAlgebraWithPrimitiveGenerator(CombinatorialFreeModule):
    r"""
    This class illustrates an implementation of a graded Hopf algebra
    with basis that has one primitive generator of degree 1 and basis
    elements indexed by non-negative integers.

    This Hopf algebra example differs from what topologists refer to as
    a graded Hopf algebra because the twist operation in the tensor rule
    satisfies

    .. MATH::

        (\mu \otimes \mu) \circ (id \otimes \tau \otimes id) \circ
        (\Delta \otimes \Delta) = \Delta \circ \mu

    where `\tau(x\otimes y) = y\otimes x`.

    """
    def __init__(self, base_ring):
        """
        EXAMPLES::

            sage: H = GradedHopfAlgebrasWithBasis(QQ).Connected().example()
            sage: TestSuite(H).run()

        """
        CombinatorialFreeModule.__init__(self, base_ring, NonNegativeIntegers(),
                                         category=GradedHopfAlgebrasWithBasis(base_ring).Connected())

    @cached_method
    def one_basis(self):
        """
        Returns 0, which index the unit of the Hopf algebra.

        OUTPUT:

        - the non-negative integer 0

        EXAMPLES::

            sage: H = GradedHopfAlgebrasWithBasis(QQ).Connected().example()
            sage: H.one_basis()
            0
            sage: H.one()
            P0

        """
        return self.basis().keys()(0)

    def degree_on_basis(self, i):
        """
        The degree of a non-negative integer is itself

        INPUT:

        - ``i`` -- a non-negative integer

        OUTPUT:

        - a non-negative integer

        TESTS::

            sage: H = GradedHopfAlgebrasWithBasis(QQ).Connected().example()
            sage: H.degree_on_basis(45)
            45

        """
        return i

    def _repr_(self):
        """
        Representation of the graded connected Hopf algebra

        EXAMPLES::

            sage: GradedHopfAlgebrasWithBasis(QQ).Connected().example()
            An example of a graded connected Hopf algebra with basis over Rational Field

        """
        return "An example of a graded connected Hopf algebra with basis over %s" % self.base_ring()

    def _repr_term(self, i):
        """
        Representation for the basis element indexed by the integer ``i``.

        EXAMPLES::

            sage: H = GradedHopfAlgebrasWithBasis(QQ).Connected().example()
            sage: H._repr_term(45)
            'P45'

        """
        return 'P' + repr(i)

    def product_on_basis(self, i, j):
        """
        The product of two basis elements.

        The product of elements of degree ``i`` and ``j`` is an element
        of degree ``i+j``.

        INPUT:

        - ``i``, ``j`` -- non-negative integers

        OUTPUT:

        - a basis element indexed by ``i+j``

        TESTS::

            sage: H = GradedHopfAlgebrasWithBasis(QQ).Connected().example()
            sage: H.monomial(4) * H.monomial(5)
            P9

        """
        return self.monomial(i+j)

    def coproduct_on_basis(self, i):
        r"""
        The coproduct of a basis element.

        .. MATH::

            \Delta(P_i) = \sum_{j=0}^i P_{i-j} \otimes P_j

        INPUT:

        - ``i`` -- a non-negative integer

        OUTPUT:

        - an element of the tensor square of ``self``

        TESTS::

            sage: H = GradedHopfAlgebrasWithBasis(QQ).Connected().example()
            sage: H.monomial(3).coproduct()
            P0 # P3 + 3*P1 # P2 + 3*P2 # P1 + P3 # P0
        """
        return self.sum_of_terms(
            ((i-j, j), binomial(i, j))
            for j in range(i+1)
        )


Example = GradedConnectedCombinatorialHopfAlgebraWithPrimitiveGenerator



# sage_setup: distribution = sagemath-categories
r"""
Finite dimensional semisimple algebras with basis
"""
#*****************************************************************************
#  Copyright (C) 2011-2015 Nicolas M. Thiery <nthiery at users.sf.net>
#                2014-2015 Aladin Virmaux <aladin.virmaux at u-psud.fr>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#                  http://www.gnu.org/licenses/
#******************************************************************************

from sage.categories.algebras import Algebras
from sage.categories.category_with_axiom import CategoryWithAxiom_over_base_ring
from sage.categories.semisimple_algebras import SemisimpleAlgebras
from sage.misc.cachefunc import cached_method


class FiniteDimensionalSemisimpleAlgebrasWithBasis(CategoryWithAxiom_over_base_ring):
    """
    The category of finite dimensional semisimple algebras with a distinguished basis

    EXAMPLES::

        sage: from sage.categories.finite_dimensional_semisimple_algebras_with_basis import FiniteDimensionalSemisimpleAlgebrasWithBasis
        sage: C = FiniteDimensionalSemisimpleAlgebrasWithBasis(QQ); C
        Category of finite dimensional semisimple algebras with basis over Rational Field

    This category is best constructed as::

        sage: D = Algebras(QQ).Semisimple().FiniteDimensional().WithBasis(); D
        Category of finite dimensional semisimple algebras with basis over Rational Field
        sage: D is C
        True

    TESTS::

        sage: TestSuite(C).run()
    """
    _base_category_class_and_axiom = (SemisimpleAlgebras.FiniteDimensional, "WithBasis")

    class ParentMethods:
        # This is needed to override the one in finite_dimensional_algebras_with_basis
        def radical_basis(self, **keywords):
            r"""
            Return a basis of the Jacobson radical of this algebra.

            - ``keywords`` -- for compatibility; ignored.

            OUTPUT: the empty list since this algebra is semisimple.

            EXAMPLES::

                sage: A = SymmetricGroup(4).algebra(QQ)                                 # needs sage.combinat sage.groups sage.modules
                sage: A.radical_basis()                                                 # needs sage.combinat sage.groups sage.modules
                ()

            TESTS::

                sage: A.radical_basis.__module__                                        # needs sage.combinat sage.groups sage.modules
                'sage.categories.finite_dimensional_semisimple_algebras_with_basis'
            """
            return ()

        @cached_method
        def central_orthogonal_idempotents(self):
            r"""
            Return a maximal list of central orthogonal
            idempotents of ``self``.

            *Central orthogonal idempotents* of an algebra `A`
            are idempotents `(e_1, \ldots, e_n)` in the center
            of `A` such that `e_i e_j = 0` whenever `i \neq j`.

            With the maximality condition, they sum up to `1`
            and are uniquely determined (up to order).

            EXAMPLES:

            For the algebra of the (abelian) alternating group `A_3`,
            we recover three idempotents corresponding to the three
            one-dimensional representations `V_i` on which `(1,2,3)`
            acts on `V_i` as multiplication by the `i`-th power of a
            cube root of unity::

                sage: # needs sage.groups sage.rings.number_field
                sage: R = CyclotomicField(3)
                sage: A3 = AlternatingGroup(3).algebra(R)
                sage: idempotents = A3.central_orthogonal_idempotents()
                sage: idempotents
                (1/3*() + 1/3*(1,2,3) + 1/3*(1,3,2),
                 1/3*() - (1/3*zeta3+1/3)*(1,2,3) - (-1/3*zeta3)*(1,3,2),
                 1/3*() - (-1/3*zeta3)*(1,2,3) - (1/3*zeta3+1/3)*(1,3,2))
                sage: A3.is_identity_decomposition_into_orthogonal_idempotents(idempotents)
                True

            For the semisimple quotient of a quiver algebra,
            we recover the vertices of the quiver::

                sage: # needs sage.graphs sage.modules sage.rings.number_field
                sage: A = FiniteDimensionalAlgebrasWithBasis(QQ).example(); A
                An example of a finite dimensional algebra with basis:
                the path algebra of the Kronecker quiver (containing
                the arrows a:x->y and b:x->y) over Rational Field
                sage: Aquo = A.semisimple_quotient()
                sage: Aquo.central_orthogonal_idempotents()
                (B['x'], B['y'])
            """
            return tuple([x.lift()
                          for x in self.center().central_orthogonal_idempotents()])

    class Commutative(CategoryWithAxiom_over_base_ring):

        class ParentMethods:

            @cached_method
            def _orthogonal_decomposition(self, generators=None):
                r"""
                Return a maximal list of orthogonal quasi-idempotents of
                this finite dimensional semisimple commutative algebra.

                INPUT:

                - ``generators`` -- a list of generators of
                  ``self`` (default: the basis of ``self``)

                OUTPUT:

                A list of quasi-idempotent elements of ``self``.

                Each quasi-idempotent `e` spans a one
                dimensional (non unital) subalgebra of
                ``self``, and cannot be decomposed as a sum
                `e=e_1+e_2` of quasi-idempotents elements.
                All together, they form a basis of ``self``.

                Up to the order and scalar factors, the result
                is unique. In particular it does not depend on
                the provided generators which are only used
                for improved efficiency.

                ALGORITHM:

                Thanks to Schur's Lemma, a commutative
                semisimple algebra `A` is a direct sum of
                dimension 1 subalgebras. The algorithm is
                recursive and proceeds as follows:

                0. If `A` is of dimension 1, return a non zero
                   element.

                1. Otherwise: find one of the generators such
                   that the morphism `x \mapsto ax` has at
                   least two (right) eigenspaces.

                2. Decompose both eigenspaces recursively.

                EXAMPLES:

                We compute an orthogonal decomposition of the
                center of the algebra of the symmetric group
                `S_4`::

                    sage: Z4 = SymmetricGroup(4).algebra(QQ).center()                   # needs sage.combinat sage.groups sage.modules
                    sage: Z4._orthogonal_decomposition()                                # needs sage.combinat sage.groups sage.modules
                    (B[0] + B[1] + B[2] + B[3] + B[4],
                     B[0] + 1/3*B[1] - 1/3*B[2] - 1/3*B[4],
                     B[0] + B[2] - 1/2*B[3],
                     B[0] - 1/3*B[1] - 1/3*B[2] + 1/3*B[4],
                     B[0] - B[1] + B[2] + B[3] - B[4])

                .. TODO::

                    Improve speed by using matrix operations
                    only, or even better delegating to a
                    multivariate polynomial solver.
                """
                if self.dimension() == 1:
                    return self.basis().list()

                category = Algebras(self.base_ring()).Semisimple().WithBasis().FiniteDimensional().Commutative().Subobjects()

                if generators is None:
                    generators = self.basis().list()

                # Searching for a good generator ...
                for gen in generators:
                    # Computing the eigenspaces of the
                    # linear map x -> gen*x
                    phi = self.module_morphism(
                        on_basis=lambda i:
                        gen*self.term(i),
                        codomain=self)
                    eigenspaces = phi.matrix().eigenspaces_right()

                    if len(eigenspaces) >= 2:
                        # Gotcha! Let's split the algebra according to the eigenspaces
                        subalgebras = [
                            self.submodule(map(self.from_vector, eigenspace.basis()),
                                           category=category)
                            for eigenvalue, eigenspace in eigenspaces]

                        # Decompose recursively each eigenspace
                        return tuple([idempotent.lift()
                                      for subalgebra in subalgebras
                                      for idempotent in subalgebra._orthogonal_decomposition()])
                # TODO: Should this be an assertion check?
                raise Exception("Unable to fully decompose %s!" % self)

            @cached_method
            def central_orthogonal_idempotents(self):
                r"""
                Return the central orthogonal idempotents of
                this semisimple commutative algebra.

                Those idempotents form a maximal decomposition
                of the identity into primitive orthogonal
                idempotents.

                OUTPUT:

                A list of orthogonal idempotents of ``self``.

                EXAMPLES::

                    sage: # needs sage.combinat sage.groups sage.modules
                    sage: A4 = SymmetricGroup(4).algebra(QQ)
                    sage: Z4 = A4.center()
                    sage: idempotents = Z4.central_orthogonal_idempotents()
                    sage: idempotents
                    (1/24*B[0] + 1/24*B[1] + 1/24*B[2] + 1/24*B[3] + 1/24*B[4],
                     3/8*B[0] + 1/8*B[1] - 1/8*B[2] - 1/8*B[4],
                     1/6*B[0] + 1/6*B[2] - 1/12*B[3],
                     3/8*B[0] - 1/8*B[1] - 1/8*B[2] + 1/8*B[4],
                     1/24*B[0] - 1/24*B[1] + 1/24*B[2] + 1/24*B[3] - 1/24*B[4])

                Lifting those idempotents from the center, we
                recognize among them the sum and alternating
                sum of all permutations::

                    sage: [e.lift() for e in idempotents]                               # needs sage.combinat sage.groups sage.modules
                    [1/24*() + 1/24*(3,4) + 1/24*(2,3) + 1/24*(2,3,4) + 1/24*(2,4,3)
                     + 1/24*(2,4) + 1/24*(1,2) + 1/24*(1,2)(3,4) + 1/24*(1,2,3)
                     + 1/24*(1,2,3,4) + 1/24*(1,2,4,3) + 1/24*(1,2,4) + 1/24*(1,3,2)
                     + 1/24*(1,3,4,2) + 1/24*(1,3) + 1/24*(1,3,4) + 1/24*(1,3)(2,4)
                     + 1/24*(1,3,2,4) + 1/24*(1,4,3,2) + 1/24*(1,4,2) + 1/24*(1,4,3)
                     + 1/24*(1,4) + 1/24*(1,4,2,3) + 1/24*(1,4)(2,3),
                     ...,
                     1/24*() - 1/24*(3,4) - 1/24*(2,3) + 1/24*(2,3,4) + 1/24*(2,4,3)
                     - 1/24*(2,4) - 1/24*(1,2) + 1/24*(1,2)(3,4) + 1/24*(1,2,3)
                     - 1/24*(1,2,3,4) - 1/24*(1,2,4,3) + 1/24*(1,2,4) + 1/24*(1,3,2)
                     - 1/24*(1,3,4,2) - 1/24*(1,3) + 1/24*(1,3,4) + 1/24*(1,3)(2,4)
                     - 1/24*(1,3,2,4) - 1/24*(1,4,3,2) + 1/24*(1,4,2) + 1/24*(1,4,3)
                     - 1/24*(1,4) - 1/24*(1,4,2,3) + 1/24*(1,4)(2,3)]

                We check that they indeed form a decomposition
                of the identity of `Z_4` into orthogonal idempotents::

                    sage: # needs sage.combinat sage.groups sage.modules
                    sage: Z4.is_identity_decomposition_into_orthogonal_idempotents(idempotents)
                    True
                """
                return tuple([(e.leading_coefficient()/(e*e).leading_coefficient())*e
                              for e in self._orthogonal_decomposition()])



# sage_setup: distribution = sagemath-categories
# sage.doctest: needs sage.combinat sage.graphs
r"""
Example of a crystal
"""
# ****************************************************************************
#  Copyright (C) 2010 Anne Schilling <anne at math.ucdavis.edu>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#                  https://www.gnu.org/licenses/
# *****************************************************************************
from sage.structure.parent import Parent
from sage.structure.element_wrapper import ElementWrapper
from sage.structure.unique_representation import UniqueRepresentation
from sage.categories.classical_crystals import ClassicalCrystals
from sage.graphs.digraph import DiGraph
from sage.categories.enumerated_sets import EnumeratedSets
from sage.combinat.root_system.cartan_type import CartanType


class HighestWeightCrystalOfTypeA(UniqueRepresentation, Parent):
    r"""
    An example of a crystal: the highest weight crystal of type `A_n`
    of highest weight `\omega_1`.

    The purpose of this class is to provide a minimal template for
    implementing crystals. See
    :class:`~sage.combinat.crystals.letters.CrystalOfLetters` for a
    full featured and optimized implementation.

    EXAMPLES::

        sage: C = Crystals().example()
        sage: C
        Highest weight crystal of type A_3 of highest weight omega_1
        sage: C.category()
        Category of classical crystals

    The elements of this crystal are in the set `\{1,\ldots,n+1\}`::

        sage: C.list()
        [1, 2, 3,  4]
        sage: C.module_generators[0]
        1

    The crystal operators themselves correspond to the elementary
    transpositions::

        sage: b = C.module_generators[0]
        sage: b.f(1)
        2
        sage: b.f(1).e(1) == b
        True

    Only the following basic operations are implemented:

    - :meth:`~sage.categories.crystals.Crystals.cartan_type` or an attribute _cartan_type
    - an attribute module_generators
    - :meth:`.Element.e`
    - :meth:`.Element.f`

    All the other usual crystal operations are inherited from the
    categories; for example::

        sage: C.cardinality()
        4

    TESTS::

        sage: C = Crystals().example()
        sage: TestSuite(C).run(verbose = True)
        running ._test_an_element() . . . pass
        running ._test_cardinality() . . . pass
        running ._test_category() . . . pass
        running ._test_construction() . . . pass
        running ._test_elements() . . .
          Running the test suite of self.an_element()
          running ._test_category() . . . pass
          running ._test_eq() . . . pass
          running ._test_new() . . . pass
          running ._test_not_implemented_methods() . . . pass
          running ._test_pickling() . . . pass
          running ._test_stembridge_local_axioms() . . . pass
          pass
        running ._test_elements_eq_reflexive() . . . pass
        running ._test_elements_eq_symmetric() . . . pass
        running ._test_elements_eq_transitive() . . . pass
        running ._test_elements_neq() . . . pass
        running ._test_enumerated_set_contains() . . . pass
        running ._test_enumerated_set_iter_cardinality() . . . pass
        running ._test_enumerated_set_iter_list() . . . pass
        running ._test_eq() . . . pass
        running ._test_fast_iter() . . . pass
        running ._test_new() . . . pass
        running ._test_not_implemented_methods() . . . pass
        running ._test_pickling() . . . pass
        running ._test_some_elements() . . . pass
        running ._test_stembridge_local_axioms() . . . pass
    """

    def __init__(self, n=3):
        """
        EXAMPLES::

            sage: C = sage.categories.examples.crystals.HighestWeightCrystalOfTypeA(n=4)
            sage: C == Crystals().example(n=4)
            True
        """
        Parent.__init__(self, category=ClassicalCrystals())
        self.n = n
        self._cartan_type = CartanType(['A', n])
        self.module_generators = [self(1)]

    def _repr_(self):
        """
        EXAMPLES::

            sage: Crystals().example()
            Highest weight crystal of type A_3 of highest weight omega_1
        """
        return "Highest weight crystal of type A_%s of highest weight omega_1" % (self.n)

    # temporary workaround while an_element is overridden by Parent
    _an_element_ = EnumeratedSets.ParentMethods._an_element_

    class Element(ElementWrapper):

        def e(self, i):
            r"""
            Returns the action of `e_i` on ``self``.

            EXAMPLES::

                sage: C = Crystals().example(4)
                sage: [[c,i,c.e(i)] for i in C.index_set() for c in C if c.e(i) is not None]
                [[2, 1, 1], [3, 2, 2], [4, 3, 3], [5, 4, 4]]
            """
            assert i in self.index_set()
            if self.value == i+1:
                return self.parent()(self.value-1)
            else:
                return None

        def f(self, i):
            r"""
            Returns the action of `f_i` on ``self``.

            EXAMPLES::

                sage: C = Crystals().example(4)
                sage: [[c,i,c.f(i)] for i in C.index_set() for c in C if c.f(i) is not None]
                [[1, 1, 2], [2, 2, 3], [3, 3, 4], [4, 4, 5]]
            """
            assert i in self.index_set()
            if self.value == i:
                return self.parent()(self.value+1)
            else:
                return None


class NaiveCrystal(UniqueRepresentation, Parent):
    r"""
    This is an example of a "crystal" which does not come from any kind of
    representation, designed primarily to test the Stembridge local rules with.
    The crystal has vertices labeled 0 through 5, with 0 the highest weight.

    The code here could also possibly be generalized to create a class that
    automatically builds a crystal from an edge-colored digraph, if someone
    feels adventurous.

    Currently, only the methods :meth:`highest_weight_vector`, :meth:`e`, and :meth:`f` are
    guaranteed to work.

    EXAMPLES::

        sage: C = Crystals().example(choice='naive')
        sage: C.highest_weight_vector()
        0
    """
    def __init__(self):
        """
        EXAMPLES::

            sage: C = sage.categories.examples.crystals.NaiveCrystal()
            sage: C == Crystals().example(choice='naive')
            True
        """
        Parent.__init__(self, category=ClassicalCrystals())
        self.n = 2
        self._cartan_type = CartanType(['A', 2])
        self.G = DiGraph(5)
        self.G.add_edges([ [0,1,1], [1,2,1], [2,3,1], [3,5,1],  [0,4,2], [4,5,2] ])
        self.module_generators = [self(0)]

    def __repr__(self):
        """
        EXAMPLES::

            sage: Crystals().example(choice='naive')
            A broken crystal, defined by digraph, of dimension five.
        """
        return "A broken crystal, defined by digraph, of dimension five."

    class Element(ElementWrapper):
        def e(self, i):
            r"""
            Returns the action of `e_i` on ``self``.

            EXAMPLES::

                sage: C = Crystals().example(choice='naive')
                sage: [[c,i,c.e(i)] for i in C.index_set() for c in [C(j) for j in [0..5]] if c.e(i) is not None]
                [[1, 1, 0], [2, 1, 1], [3, 1, 2], [5, 1, 3], [4, 2, 0], [5, 2, 4]]
            """
            assert i in self.index_set()
            for edge in self.parent().G.edges(sort=False):
                if edge[1] == int(str(self)) and edge[2] == i:
                    return self.parent()(edge[0])
            return None

        def f(self, i):
            r"""
            Returns the action of `f_i` on ``self``.

            EXAMPLES::

                sage: C = Crystals().example(choice='naive')
                sage: [[c,i,c.f(i)] for i in C.index_set() for c in [C(j) for j in [0..5]] if c.f(i) is not None]
                [[0, 1, 1], [1, 1, 2], [2, 1, 3], [3, 1, 5], [0, 2, 4], [4, 2, 5]]
            """
            assert i in self.index_set()
            for edge in self.parent().G.edges_incident(int(str(self))):
                if edge[2] == i:
                    return self.parent()(edge[1])
            return None



# sage_setup: distribution = sagemath-categories
r"""
Super modules
"""
#*****************************************************************************
#  Copyright (C) 2015 Travis Scrimshaw <tscrim at ucdavis.edu>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#                  http://www.gnu.org/licenses/
#******************************************************************************

from sage.categories.category_types import Category_over_base_ring
from sage.categories.covariant_functorial_construction import CovariantConstructionCategory

# Note, a commutative algebra is not a commutative super algebra,
#   therefore the following whitelist.
axiom_whitelist = frozenset(["Facade", "Finite", "Infinite",
                             "FiniteDimensional", "Connected", "WithBasis",
                             "FinitelyGeneratedAsLambdaBracketAlgebra",
                             # "Commutative", "Cocommutative",
                             "Supercommutative", "Supercocommutative",
                             "Associative", "Inverse", "Unital", "Division",
                             "AdditiveCommutative", "AdditiveAssociative",
                             "AdditiveInverse", "AdditiveUnital",
                             "NoZeroDivisors", "Distributive"])


class SuperModulesCategory(CovariantConstructionCategory, Category_over_base_ring):
    @classmethod
    def default_super_categories(cls, category, *args):
        """
        Return the default super categories of `F_{Cat}(A,B,...)` for
        `A,B,...` parents in `Cat`.

        INPUT:

        - ``cls`` -- the category class for the functor `F`
        - ``category`` -- a category `Cat`
        - ``*args`` -- further arguments for the functor

        OUTPUT:

        A join category.

        This implements the property that subcategories constructed by
        the set of whitelisted axioms is a subcategory.

        EXAMPLES::

            sage: HopfAlgebras(ZZ).WithBasis().FiniteDimensional().Super()  # indirect doctest
            Category of finite dimensional super Hopf algebras with basis over Integer Ring
        """
        axioms = axiom_whitelist.intersection(category.axioms())
        C = super().default_super_categories(category, *args)
        return C._with_axioms(axioms)

    def __init__(self, base_category):
        """
        EXAMPLES::

            sage: C = Algebras(QQ).Super()
            sage: C
            Category of super algebras over Rational Field
            sage: C.base_category()
            Category of algebras over Rational Field
            sage: sorted(C.super_categories(), key=str)
            [Category of graded algebras over Rational Field,
             Category of super modules over Rational Field]

            sage: AlgebrasWithBasis(QQ).Super().base_ring()
            Rational Field
            sage: HopfAlgebrasWithBasis(QQ).Super().base_ring()
            Rational Field
        """
        super().__init__(base_category, base_category.base_ring())

    _functor_category = "Super"

    def _repr_object_names(self):
        """
        EXAMPLES::

            sage: AlgebrasWithBasis(QQ).Super()  # indirect doctest
            Category of super algebras with basis over Rational Field
        """
        return "super {}".format(self.base_category()._repr_object_names())


class SuperModules(SuperModulesCategory):
    r"""
    The category of super modules.

    An `R`-*super module* (where `R` is a ring) is an `R`-module `M` equipped
    with a decomposition `M = M_0 \oplus M_1` into two `R`-submodules
    `M_0` and `M_1` (called the *even part* and the *odd part* of `M`,
    respectively).

    Thus, an `R`-super module automatically becomes a `\ZZ / 2 \ZZ`-graded
    `R`-module, with `M_0` being the degree-`0` component and `M_1` being the
    degree-`1` component.

    EXAMPLES::

        sage: Modules(ZZ).Super()
        Category of super modules over Integer Ring
        sage: Modules(ZZ).Super().super_categories()
        [Category of graded modules over Integer Ring]

    The category of super modules defines the super structure which
    shall be preserved by morphisms::

        sage: Modules(ZZ).Super().additional_structure()
        Category of super modules over Integer Ring

    TESTS::

        sage: TestSuite(Modules(ZZ).Super()).run()
    """
    def super_categories(self):
        """
        EXAMPLES::

            sage: Modules(ZZ).Super().super_categories()
            [Category of graded modules over Integer Ring]

        Nota bene::

            sage: Modules(QQ).Super()
            Category of super modules over Rational Field
            sage: Modules(QQ).Super().super_categories()
            [Category of graded modules over Rational Field]
        """
        return [self.base_category().Graded()]

    def extra_super_categories(self):
        r"""
        Adds :class:`VectorSpaces` to the super categories of ``self`` if
        the base ring is a field.

        EXAMPLES::

            sage: Modules(QQ).Super().extra_super_categories()
            [Category of vector spaces over Rational Field]
            sage: Modules(ZZ).Super().extra_super_categories()
            []

        This makes sure that ``Modules(QQ).Super()`` returns an
        instance of :class:`SuperModules` and not a join category of
        an instance of this class and of ``VectorSpaces(QQ)``::

            sage: type(Modules(QQ).Super())
            <class 'sage.categories.super_modules.SuperModules_with_category'>

        .. TODO::

            Get rid of this workaround once there is a more systematic
            approach for the alias ``Modules(QQ)`` -> ``VectorSpaces(QQ)``.
            Probably the latter should be a category with axiom, and
            covariant constructions should play well with axioms.
        """
        from sage.categories.modules import Modules
        from sage.categories.fields import Fields
        base_ring = self.base_ring()
        if base_ring in Fields():
            return [Modules(base_ring)]
        else:
            return []

    class ParentMethods:
        pass

    class ElementMethods:
        def is_even_odd(self):
            """
            Return ``0`` if ``self`` is an even element or ``1``
            if an odd element.

            .. NOTE::

                The default implementation assumes that the even/odd is
                determined by the parity of :meth:`degree`.

                Overwrite this method if the even/odd behavior is desired
                to be independent.

            EXAMPLES::

                sage: # needs sage.combinat sage.modules
                sage: cat = Algebras(QQ).WithBasis().Super()
                sage: C = CombinatorialFreeModule(QQ, Partitions(), category=cat)
                sage: C.degree_on_basis = sum
                sage: C.basis()[2,2,1].is_even_odd()
                1
                sage: C.basis()[2,2].is_even_odd()
                0
            """
            return self.degree() % 2

        def is_even(self):
            """
            Return if ``self`` is an even element.

            EXAMPLES::

                sage: # needs sage.combinat sage.modules
                sage: cat = Algebras(QQ).WithBasis().Super()
                sage: C = CombinatorialFreeModule(QQ, Partitions(), category=cat)
                sage: C.degree_on_basis = sum
                sage: C.basis()[2,2,1].is_even()
                False
                sage: C.basis()[2,2].is_even()
                True
            """
            return self.is_even_odd() == 0

        def is_odd(self):
            """
            Return if ``self`` is an odd element.

            EXAMPLES::

                sage: # needs sage.combinat sage.modules
                sage: cat = Algebras(QQ).WithBasis().Super()
                sage: C = CombinatorialFreeModule(QQ, Partitions(), category=cat)
                sage: C.degree_on_basis = sum
                sage: C.basis()[2,2,1].is_odd()
                True
                sage: C.basis()[2,2].is_odd()
                False
            """
            return self.is_even_odd() == 1



#!/usr/bin/env python

from distutils import log
from setuptools import setup

# Work around a Cython problem in Python 3.8.x on macOS
# https://github.com/cython/cython/issues/3262
import os
if os.uname().sysname == 'Darwin':
    import multiprocessing
    multiprocessing.set_start_method('fork', force=True)

# PEP 517 builds do not have . in sys.path
import sys
sys.path.insert(0, os.path.dirname(__file__))

from sage_setup.excepthook import excepthook
sys.excepthook = excepthook

from sage_setup.setenv import setenv
setenv()

import sage.env
sage.env.default_required_modules = sage.env.default_optional_modules = ()

from sage_setup.command.sage_build_cython import sage_build_cython
from sage_setup.command.sage_build_ext import sage_build_ext

cmdclass = dict(build_cython=sage_build_cython,
                build_ext=sage_build_ext)

from sage_setup.find import find_python_sources
python_packages, python_modules, cython_modules = find_python_sources(
    '.', ['sage'])   # for now, we do the filtering using MANIFEST

log.warn('python_packages = {0}'.format(python_packages))
log.warn('python_modules = {0}'.format(python_modules))
log.warn('cython_modules = {0}'.format(cython_modules))

setup(
    cmdclass = cmdclass,
    packages = python_packages,
    py_modules  = python_modules,
    ext_modules = cython_modules,
)



# sage_setup: distribution = sagemath-categories
# sage.doctest: needs sage.combinat sage.modules
r"""
Examples of a Lie algebra with basis
"""
#*****************************************************************************
#  Copyright (C) 2014 Travis Scrimshaw <tscrim at ucdavis.edu>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#                  http://www.gnu.org/licenses/
#*****************************************************************************

#from sage.misc.cachefunc import cached_method
from sage.sets.family import Family
from sage.categories.lie_algebras import LieAlgebras
from sage.categories.algebras import Algebras
from sage.monoids.indexed_free_monoid import IndexedFreeAbelianMonoid
from sage.combinat.free_module import CombinatorialFreeModule


class AbelianLieAlgebra(CombinatorialFreeModule):
    r"""
    An example of a Lie algebra: the abelian Lie algebra.

    This class illustrates a minimal implementation of a Lie algebra with
    a distinguished basis.
    """
    def __init__(self, R, gens):
        """
        EXAMPLES::

            sage: L = LieAlgebras(QQ).WithBasis().example()
            sage: TestSuite(L).run()
        """
        cat = LieAlgebras(R).WithBasis()
        CombinatorialFreeModule.__init__(self, R, gens, category=cat)

    def _construct_UEA(self):
        """
        Construct the universal enveloping algebra of ``self``.

        EXAMPLES::

            sage: L = LieAlgebras(QQ).WithBasis().example()
            sage: L._construct_UEA()
            Polynomial algebra with generators indexed by Partitions over Rational Field
        """
        return IndexedPolynomialRing(self.base_ring(), self._indices)

    def _repr_(self):
        """
        EXAMPLES::

            sage: LieAlgebras(QQ).WithBasis().example()
            An example of a Lie algebra: the abelian Lie algebra on the
             generators indexed by Partitions over Rational Field
        """
        return "An example of a Lie algebra: the abelian Lie algebra on the" \
               " generators indexed by {} over {}".format(
                        self.basis().keys(), self.base_ring())

    def lie_algebra_generators(self):
        """
        Return the generators of ``self`` as a Lie algebra.

        EXAMPLES::

            sage: L = LieAlgebras(QQ).WithBasis().example()
            sage: L.lie_algebra_generators()
            Lazy family (Term map from Partitions to
             An example of a Lie algebra: the abelian Lie algebra on the
             generators indexed by Partitions over Rational
             Field(i))_{i in Partitions}
        """
        return self.basis()

    def bracket_on_basis(self, x, y):
        """
        Return the Lie bracket on basis elements indexed by ``x`` and ``y``.

        EXAMPLES::

            sage: L = LieAlgebras(QQ).WithBasis().example()
            sage: L.bracket_on_basis(Partition([4,1]), Partition([2,2,1]))
            0
        """
        return self.zero()

    class Element(CombinatorialFreeModule.Element):
        def lift(self):
            """
            Return the lift of ``self`` to the universal enveloping algebra.

            EXAMPLES::

                sage: L = LieAlgebras(QQ).WithBasis().example()
                sage: elt = L.an_element()
                sage: elt.lift()
                3*P[F[2]] + 2*P[F[1]] + 2*P[F[]]
            """
            UEA = self.parent().universal_enveloping_algebra()
            I = UEA._indices
            return UEA.sum_of_terms((I.gen(t), c) for t, c in self)


Example = AbelianLieAlgebra

##############


class IndexedPolynomialRing(CombinatorialFreeModule):
    """
    Polynomial ring whose generators are indexed by an arbitrary set.

    .. TODO::

        Currently this is just used as the universal enveloping algebra
        for the example of the abelian Lie algebra. This should be
        factored out into a more complete class.
    """
    def __init__(self, R, indices, **kwds):
        """
        Initialize ``self``.

        EXAMPLES::

            sage: L = LieAlgebras(QQ).WithBasis().example()
            sage: UEA = L.universal_enveloping_algebra()
            sage: TestSuite(UEA).run()
        """
        if 'category' not in kwds:
            kwds['category'] = Algebras(R).WithBasis()
        if 'prefix' not in kwds:
            kwds['prefix'] = 'P'
        # This is a workaround until IndexedFree(Abelian)Monoid elements compare properly
        kwds['sorting_key'] = lambda x: x.to_word_list()
        kwds['sorting_reverse'] = True
        M = IndexedFreeAbelianMonoid(indices, bracket='')
        CombinatorialFreeModule.__init__(self, R, M, **kwds)

    def _repr_(self):
        """
        Return a string representation of ``self``.

        EXAMPLES::

            sage: L = LieAlgebras(QQ).WithBasis().example()
            sage: L.universal_enveloping_algebra()
            Polynomial algebra with generators indexed by Partitions over Rational Field
        """
        return "Polynomial algebra with generators indexed by {} over {}".format(
            self._indices._indices, self.base_ring())

    def one_basis(self):
        """
        Return the index of element `1`.

        EXAMPLES::

            sage: L = LieAlgebras(QQ).WithBasis().example()
            sage: UEA = L.universal_enveloping_algebra()
            sage: UEA.one_basis()
            1
            sage: UEA.one_basis().parent()
            Free abelian monoid indexed by Partitions
        """
        return self._indices.one()

    def product_on_basis(self, x, y):
        """
        Return the product of the monomials indexed by ``x`` and ``y``.

        EXAMPLES::

            sage: L = LieAlgebras(QQ).WithBasis().example()
            sage: UEA = L.universal_enveloping_algebra()
            sage: I = UEA._indices
            sage: UEA.product_on_basis(I.an_element(), I.an_element())
            P[F[]^4*F[1]^4*F[2]^6]
        """
        return self.monomial(x*y)

    def algebra_generators(self):
        """
        Return the algebra generators of ``self``.

        EXAMPLES::

            sage: L = LieAlgebras(QQ).WithBasis().example()
            sage: UEA = L.universal_enveloping_algebra()
            sage: UEA.algebra_generators()
            Lazy family (algebra generator map(i))_{i in Partitions}
        """
        I = self._indices
        return Family(I._indices, lambda x: self.monomial(I.gen(x)),
                      name="algebra generator map")



# sage_setup: distribution = sagemath-categories
r"""
Manifolds
"""
#*****************************************************************************
#  Copyright (C) 2015 Travis Scrimshaw <tscrim at ucdavis.edu>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#                  http://www.gnu.org/licenses/
#******************************************************************************

from sage.misc.abstract_method import abstract_method
from sage.misc.cachefunc import cached_method
from sage.categories.category_types import Category_over_base_ring
from sage.categories.category_with_axiom import CategoryWithAxiom_over_base_ring
from sage.categories.sets_cat import Sets
from sage.categories.fields import Fields


class Manifolds(Category_over_base_ring):
    r"""
    The category of manifolds over any topological field.

    Let `k` be a topological field. A `d`-dimensional `k`-*manifold* `M`
    is a second countable Hausdorff space such that the neighborhood of
    any point `x \in M` is homeomorphic to `k^d`.

    EXAMPLES::

        sage: # needs sage.rings.real_mpfr
        sage: from sage.categories.manifolds import Manifolds
        sage: C = Manifolds(RR); C
        Category of manifolds over Real Field with 53 bits of precision
        sage: C.super_categories()
        [Category of topological spaces]

    TESTS::

        sage: TestSuite(C).run(skip="_test_category_over_bases")                        # needs sage.rings.real_mpfr
    """
    def __init__(self, base, name=None):
        r"""
        Initialize ``self``.

        EXAMPLES::

            sage: from sage.categories.manifolds import Manifolds
            sage: C = Manifolds(RR)
            sage: TestSuite(C).run(skip="_test_category_over_bases")
        """
        if base not in Fields().Topological():
            raise ValueError("base must be a topological field")
        Category_over_base_ring.__init__(self, base, name)

    @cached_method
    def super_categories(self):
        """
        EXAMPLES::

            sage: from sage.categories.manifolds import Manifolds
            sage: Manifolds(RR).super_categories()
            [Category of topological spaces]
        """
        return [Sets().Topological()]

    def additional_structure(self):
        r"""
        Return ``None``.

        Indeed, the category of manifolds defines no new
        structure: a morphism of topological spaces between
        manifolds is a manifold morphism.

        .. SEEALSO:: :meth:`Category.additional_structure`

        EXAMPLES::

            sage: from sage.categories.manifolds import Manifolds
            sage: Manifolds(RR).additional_structure()
        """
        return None

    class ParentMethods:
        @abstract_method
        def dimension(self):
            """
            Return the dimension of ``self``.

            EXAMPLES::

                sage: from sage.categories.manifolds import Manifolds
                sage: M = Manifolds(RR).example()
                sage: M.dimension()
                3
            """

    class SubcategoryMethods:
        @cached_method
        def Connected(self):
            """
            Return the full subcategory of the connected objects of ``self``.

            EXAMPLES::

                sage: from sage.categories.manifolds import Manifolds
                sage: Manifolds(RR).Connected()                                         # needs sage.rings.real_mpfr
                Category of connected manifolds
                 over Real Field with 53 bits of precision

            TESTS::

                sage: Manifolds(RR).Connected.__module__
                'sage.categories.manifolds'
            """
            return self._with_axiom('Connected')

        @cached_method
        def FiniteDimensional(self):
            """
            Return the full subcategory of the finite dimensional
            objects of ``self``.

            EXAMPLES::

                sage: from sage.categories.manifolds import Manifolds
                sage: C = Manifolds(RR).Connected().FiniteDimensional(); C              # needs sage.rings.real_mpfr
                Category of finite dimensional connected manifolds
                 over Real Field with 53 bits of precision

            TESTS::

                sage: from sage.categories.manifolds import Manifolds
                sage: Manifolds(RR).Connected().FiniteDimensional.__module__
                'sage.categories.manifolds'
            """
            return self._with_axiom('FiniteDimensional')

        @cached_method
        def Differentiable(self):
            """
            Return the subcategory of the differentiable objects
            of ``self``.

            EXAMPLES::

                sage: from sage.categories.manifolds import Manifolds
                sage: Manifolds(RR).Differentiable()                                    # needs sage.rings.real_mpfr
                Category of differentiable manifolds
                 over Real Field with 53 bits of precision

            TESTS::

                sage: TestSuite(Manifolds(RR).Differentiable()).run()
                sage: Manifolds(RR).Differentiable.__module__
                'sage.categories.manifolds'
            """
            return self._with_axiom('Differentiable')

        @cached_method
        def Smooth(self):
            """
            Return the subcategory of the smooth objects of ``self``.

            EXAMPLES::

                sage: from sage.categories.manifolds import Manifolds
                sage: Manifolds(RR).Smooth()                                            # needs sage.rings.real_mpfr
                Category of smooth manifolds
                 over Real Field with 53 bits of precision

            TESTS::

                sage: TestSuite(Manifolds(RR).Smooth()).run()
                sage: Manifolds(RR).Smooth.__module__
                'sage.categories.manifolds'
            """
            return self._with_axiom('Smooth')

        @cached_method
        def Analytic(self):
            """
            Return the subcategory of the analytic objects of ``self``.

            EXAMPLES::

                sage: from sage.categories.manifolds import Manifolds
                sage: Manifolds(RR).Analytic()                                          # needs sage.rings.real_mpfr
                Category of analytic manifolds
                 over Real Field with 53 bits of precision

            TESTS::

                sage: TestSuite(Manifolds(RR).Analytic()).run()
                sage: Manifolds(RR).Analytic.__module__
                'sage.categories.manifolds'
            """
            return self._with_axiom('Analytic')

        @cached_method
        def AlmostComplex(self):
            """
            Return the subcategory of the almost complex objects
            of ``self``.

            EXAMPLES::

                sage: from sage.categories.manifolds import Manifolds
                sage: Manifolds(RR).AlmostComplex()                                     # needs sage.rings.real_mpfr
                Category of almost complex manifolds
                 over Real Field with 53 bits of precision

            TESTS::

                sage: TestSuite(Manifolds(RR).AlmostComplex()).run()
                sage: Manifolds(RR).AlmostComplex.__module__
                'sage.categories.manifolds'
            """
            return self._with_axiom('AlmostComplex')

        @cached_method
        def Complex(self):
            r"""
            Return the subcategory of manifolds over `\CC` of ``self``.

            EXAMPLES::

                sage: from sage.categories.manifolds import Manifolds
                sage: Manifolds(CC).Complex()                                           # needs sage.rings.real_mpfr
                Category of complex manifolds over
                 Complex Field with 53 bits of precision

            TESTS::

                sage: TestSuite(Manifolds(CC).Complex()).run()                          # needs sage.rings.real_mpfr
                sage: Manifolds(CC).Complex.__module__                                  # needs sage.rings.real_mpfr
                'sage.categories.manifolds'
            """
            return ComplexManifolds(self.base())._with_axioms(self.axioms())

    class Differentiable(CategoryWithAxiom_over_base_ring):
        """
        The category of differentiable manifolds.

        A differentiable manifold is a manifold with a differentiable atlas.
        """

    class Smooth(CategoryWithAxiom_over_base_ring):
        """
        The category of smooth manifolds.

        A smooth manifold is a manifold with a smooth atlas.
        """
        def extra_super_categories(self):
            """
            Return the extra super categories of ``self``.

            A smooth manifold is differentiable.

            EXAMPLES::

                sage: from sage.categories.manifolds import Manifolds
                sage: Manifolds(RR).Smooth().super_categories()  # indirect doctest     # needs sage.rings.real_mpfr
                [Category of differentiable manifolds
                 over Real Field with 53 bits of precision]
            """
            return [Manifolds(self.base()).Differentiable()]

    class Analytic(CategoryWithAxiom_over_base_ring):
        r"""
        The category of complex manifolds.

        An analytic manifold is a manifold with an analytic atlas.
        """
        def extra_super_categories(self):
            """
            Return the extra super categories of ``self``.

            An analytic manifold is smooth.

            EXAMPLES::

                sage: from sage.categories.manifolds import Manifolds
                sage: Manifolds(RR).Analytic().super_categories()  # indirect doctest   # needs sage.rings.real_mpfr
                [Category of smooth manifolds
                 over Real Field with 53 bits of precision]
            """
            return [Manifolds(self.base()).Smooth()]

    class AlmostComplex(CategoryWithAxiom_over_base_ring):
        r"""
        The category of almost complex manifolds.

        An *almost complex manifold* `M` is a manifold with a smooth tensor
        field `J` of rank `(1, 1)` such that `J^2 = -1` when regarded as a
        vector bundle isomorphism `J : TM \to TM` on the tangent bundle.
        The tensor field `J` is called the *almost complex structure* of `M`.
        """
        def extra_super_categories(self):
            """
            Return the extra super categories of ``self``.

            An almost complex manifold is smooth.

            EXAMPLES::

                sage: from sage.categories.manifolds import Manifolds
                sage: Manifolds(RR).AlmostComplex().super_categories()  # indirect doctest          # needs sage.rings.real_mpfr
                [Category of smooth manifolds
                 over Real Field with 53 bits of precision]
            """
            return [Manifolds(self.base()).Smooth()]

    class FiniteDimensional(CategoryWithAxiom_over_base_ring):
        """
        Category of finite dimensional manifolds.

        EXAMPLES::

            sage: from sage.categories.manifolds import Manifolds
            sage: C = Manifolds(RR).FiniteDimensional()
            sage: TestSuite(C).run(skip="_test_category_over_bases")
        """

    class Connected(CategoryWithAxiom_over_base_ring):
        """
        The category of connected manifolds.

        EXAMPLES::

            sage: from sage.categories.manifolds import Manifolds
            sage: C = Manifolds(RR).Connected()
            sage: TestSuite(C).run(skip="_test_category_over_bases")
        """


class ComplexManifolds(Category_over_base_ring):
    r"""
    The category of complex manifolds.

    A `d`-dimensional complex manifold is a manifold whose underlying
    vector space is `\CC^d` and has a holomorphic atlas.
    """
    @cached_method
    def super_categories(self):
        """
        EXAMPLES::

            sage: from sage.categories.manifolds import Manifolds
            sage: Manifolds(RR).super_categories()
            [Category of topological spaces]
        """
        return [Manifolds(self.base()).Analytic()]



# sage_setup: distribution = sagemath-categories
# sage.doctest: needs sage.combinat sage.groups
r"""
Finite Coxeter Groups
"""
# ****************************************************************************
#  Copyright (C) 2009    Nicolas M. Thiery <nthiery at users.sf.net>
#  Copyright (C) 2009    Nicolas Borie <nicolas dot borie at math.u-psud.fr>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#                  https://www.gnu.org/licenses/
# *****************************************************************************

import sage.rings.abc

from sage.misc.cachefunc import cached_method, cached_in_parent_method
from sage.misc.lazy_attribute import lazy_attribute
from sage.categories.category_with_axiom import CategoryWithAxiom
from sage.categories.coxeter_groups import CoxeterGroups


class FiniteCoxeterGroups(CategoryWithAxiom):
    r"""
    The category of finite Coxeter groups.

    EXAMPLES::

        sage: CoxeterGroups.Finite()
        Category of finite Coxeter groups
        sage: FiniteCoxeterGroups().super_categories()
        [Category of finite generalized Coxeter groups,
         Category of Coxeter groups]

        sage: G = CoxeterGroups().Finite().example()
        sage: G.cayley_graph(side = "right").plot()
        Graphics object consisting of 40 graphics primitives

    Here are some further examples::

        sage: WeylGroups().Finite().example()
        The symmetric group on {0, ..., 3}

        sage: WeylGroup(["B", 3])
        Weyl Group of type ['B', 3] (as a matrix group acting on the ambient space)

    Those other examples will eventually be also in this category::

        sage: SymmetricGroup(4)
        Symmetric group of order 4! as a permutation group
        sage: DihedralGroup(5)
        Dihedral group of order 10 as a permutation group
    """
    def extra_super_categories(self):
        r"""
        EXAMPLES::

            sage: CoxeterGroups().Finite().super_categories()
            [Category of finite generalized Coxeter groups,
             Category of Coxeter groups]
        """
        from sage.categories.complex_reflection_groups import ComplexReflectionGroups
        return [ComplexReflectionGroups().Finite().WellGenerated()]

    class ParentMethods:
        """
        Ambiguity resolution: the implementation of ``some_elements``
        is preferable to that of :class:`FiniteGroups`. The same holds
        for ``__iter__``, although a breadth first search would be more
        natural; at least this maintains backward compatibility after
        :issue:`13589`.

        TESTS::

            sage: W = FiniteCoxeterGroups().example(3)

            sage: W.some_elements.__module__
            'sage.categories.complex_reflection_or_generalized_coxeter_groups'
            sage: W.__iter__.__module__
            'sage.categories.coxeter_groups'

            sage: W.some_elements()
            [(1,), (2,), (), (1, 2)]
            sage: list(W)
            [(), (1,), (2,), (1, 2), (2, 1), (1, 2, 1)]
        """
        __iter__ = CoxeterGroups.ParentMethods.__dict__["__iter__"]

        @lazy_attribute
        def w0(self):
            r"""
            Return the longest element of ``self``.

            This attribute is deprecated, use :meth:`long_element` instead.

            EXAMPLES::

                sage: D8 = FiniteCoxeterGroups().example(8)
                sage: D8.w0
                (1, 2, 1, 2, 1, 2, 1, 2)
                sage: D3 = FiniteCoxeterGroups().example(3)
                sage: D3.w0
                (1, 2, 1)
            """
            return self.long_element()

        def long_element(self, index_set=None, as_word=False):
            r"""
            Return the longest element of ``self``, or of the
            parabolic subgroup corresponding to the given ``index_set``.

            INPUT:

            - ``index_set`` -- a subset (as a list or iterable) of the
              nodes of the Dynkin diagram; (default: all of them)

            - ``as_word`` -- boolean (default ``False``). If ``True``, then
              return instead a reduced decomposition of the longest element.

            Should this method be called maximal_element? longest_element?

            EXAMPLES::

                sage: D10 = FiniteCoxeterGroups().example(10)
                sage: D10.long_element()
                (1, 2, 1, 2, 1, 2, 1, 2, 1, 2)
                sage: D10.long_element([1])
                (1,)
                sage: D10.long_element([2])
                (2,)
                sage: D10.long_element([])
                ()

                sage: D7 = FiniteCoxeterGroups().example(7)
                sage: D7.long_element()
                (1, 2, 1, 2, 1, 2, 1)

            One can require instead a reduced word for w0::

                sage: A3 = CoxeterGroup(['A', 3])
                sage: A3.long_element(as_word=True)
                [1, 2, 1, 3, 2, 1]
            """
            if index_set is None:
                index_set = self.index_set()
            w = self.one()
            if as_word:
                word = []
            while True:
                i = w.first_descent(index_set=index_set, positive=True)
                if i is None:
                    if as_word:
                        return word
                    else:
                        return w
                else:
                    if as_word:
                        word.append(i)
                    w = w.apply_simple_reflection(i)

        @cached_method
        def bruhat_poset(self, facade=False):
            """
            Return the Bruhat poset of ``self``.

            .. SEEALSO::

                :meth:`bhz_poset`, :meth:`shard_poset`, :meth:`weak_poset`

            EXAMPLES::

                sage: W = WeylGroup(["A", 2])
                sage: P = W.bruhat_poset()
                sage: P
                Finite poset containing 6 elements
                sage: P.show()

            Here are some typical operations on this poset::

                sage: W = WeylGroup(["A", 3])
                sage: P = W.bruhat_poset()
                sage: u = W.from_reduced_word([3,1])
                sage: v = W.from_reduced_word([3,2,1,2,3])
                sage: P(u) <= P(v)
                True
                sage: len(P.interval(P(u), P(v)))
                10
                sage: P.is_join_semilattice()
                False

            By default, the elements of `P` are aware that they belong
            to `P`::

                sage: P.an_element().parent()
                Finite poset containing 24 elements

            If instead one wants the elements to be plain elements of
            the Coxeter group, one can use the ``facade`` option::

                sage: P = W.bruhat_poset(facade = True)
                sage: P.an_element().parent()
                Weyl Group of type ['A', 3] (as a matrix group acting on the ambient space)

            .. SEEALSO:: :func:`Poset` for more on posets and facade posets.

            TESTS::

                sage: [len(WeylGroup(["A", n]).bruhat_poset().cover_relations()) for n in [1,2,3]]
                [1, 8, 58]

            .. TODO::

                - Use the symmetric group in the examples (for nicer
                  output), and print the edges for a stronger test.
                - The constructed poset should be lazy, in order to
                  handle large / infinite Coxeter groups.
            """
            from sage.combinat.posets.posets import Poset
            covers = tuple([u, v] for v in self for u in v.bruhat_lower_covers())
            return Poset((self, covers), cover_relations=True, facade=facade)

        def shard_poset(self, side='right'):
            """
            Return the shard intersection order attached to `W`.

            This is a lattice structure on `W`, introduced in [Rea2009]_. It
            contains the noncrossing partition lattice, as the induced lattice
            on the subset of `c`-sortable elements.

            The partial order is given by simultaneous inclusion of inversion sets
            and subgroups attached to every element.

            The precise description used here can be found in [STW2018]_.

            Another implementation for the symmetric groups is
            available as :func:`~sage.combinat.shard_order.shard_poset`.

            .. SEEALSO::

                :meth:`bhz_poset`, :meth:`bruhat_poset`, :meth:`weak_poset`

            EXAMPLES::

                sage: W = CoxeterGroup(['A',3], base_ring=ZZ)
                sage: SH = W.shard_poset(); SH
                Finite lattice containing 24 elements
                sage: SH.is_graded()
                True
                sage: SH.characteristic_polynomial()
                q^3 - 11*q^2 + 23*q - 13
                sage: SH.f_polynomial()
                34*q^3 + 22*q^2 + q
            """
            from sage.combinat.posets.lattices import LatticePoset
            data = {w: (frozenset(u.lift()
                                  for u in w.covered_reflections_subgroup()),
                        frozenset((~w).inversions_as_reflections()))
                    for w in self}

            def shard_comparison(u, v):
                Gu, Nu = data[u]
                Gv, Nv = data[v]
                return Gu.issubset(Gv) and Nu.issubset(Nv)

            return LatticePoset([self, shard_comparison])

        def bhz_poset(self):
            r"""
            Return the Bergeron-Hohlweg-Zabrocki partial order on the Coxeter
            group.

            This is a partial order on the elements of a finite
            Coxeter group `W`, which is distinct from the Bruhat
            order, the weak order and the shard intersection order. It
            was defined in [BHZ2005]_.

            This partial order is not a lattice, as there is no unique
            maximal element. It can be succintly defined as follows.

            Let `u` and `v` be two elements of the Coxeter group `W`. Let
            `S(u)` be the support of `u`. Then `u \leq v` if and only
            if `v_{S(u)} = u` (here `v = v^I v_I` denotes the usual
            parabolic decomposition with respect to the standard parabolic
            subgroup `W_I`).

            .. SEEALSO::

                :meth:`bruhat_poset`, :meth:`shard_poset`, :meth:`weak_poset`

            EXAMPLES::

                sage: W = CoxeterGroup(['A',3], base_ring=ZZ)
                sage: P = W.bhz_poset(); P
                Finite poset containing 24 elements
                sage: P.relations_number()
                103
                sage: P.chain_polynomial()
                34*q^4 + 90*q^3 + 79*q^2 + 24*q + 1
                sage: len(P.maximal_elements())
                13
            """
            from sage.graphs.digraph import DiGraph
            from sage.combinat.posets.posets import Poset

            def covered_by(ux, vy):
                u, iu, Su = ux
                v, iv, Sv = vy
                if len(Sv) != len(Su) + 1:
                    return False
                if not all(u in Sv for u in Su):
                    return False
                return all((v * iu).has_descent(x, positive=True) for x in Su)

            vertices = [(u, u.inverse(),
                         tuple(set(u.reduced_word_reverse_iterator())))
                        for u in self]
            dg = DiGraph([vertices, covered_by])
            dg.relabel(lambda x: x[0])
            return Poset(dg, cover_relations=True)

        def degrees(self):
            """
            Return the degrees of the Coxeter group.

            The output is an increasing list of integers.

            EXAMPLES::

                sage: CoxeterGroup(['A', 4]).degrees()
                (2, 3, 4, 5)
                sage: CoxeterGroup(['B', 4]).degrees()
                (2, 4, 6, 8)
                sage: CoxeterGroup(['D', 4]).degrees()
                (2, 4, 4, 6)
                sage: CoxeterGroup(['F', 4]).degrees()
                (2, 6, 8, 12)
                sage: CoxeterGroup(['E', 8]).degrees()
                (2, 8, 12, 14, 18, 20, 24, 30)
                sage: CoxeterGroup(['H', 3]).degrees()
                (2, 6, 10)

                sage: WeylGroup([["A",3], ["A",3], ["B",2]]).degrees()
                (2, 3, 4, 2, 3, 4, 2, 4)

            TESTS::

                sage: CoxeterGroup(['A', 4]).degrees()
                (2, 3, 4, 5)
                sage: SymmetricGroup(3).degrees()
                (2, 3)
            """
            from sage.rings.qqbar import QQbar
            from sage.rings.integer_ring import ZZ

            def degrees_of_irreducible_component(I):
                """Return the degrees for the irreducible component indexed by I"""
                # A Coxeter element
                s = self.simple_reflections()
                c = self.prod(s[i] for i in I)
                roots = c.matrix().change_ring(QQbar).charpoly().roots()
                args = ((z.rational_argument(), m) for z, m in roots)
                args = [(z if z >= 0 else 1 + z, m) for z, m in args]
                h = max(z.denominator() for z, m in args)
                return tuple(sorted(ZZ(z * h + 1)
                                    for z, m in args if z
                                    for i in range(m)))

            return sum((degrees_of_irreducible_component(I)
                        for I in self.irreducible_component_index_sets()), ())

        def codegrees(self):
            """
            Return the codegrees of the Coxeter group.

            These are just the degrees minus 2.

            EXAMPLES::

                sage: CoxeterGroup(['A', 4]).codegrees()
                (0, 1, 2, 3)
                sage: CoxeterGroup(['B', 4]).codegrees()
                (0, 2, 4, 6)
                sage: CoxeterGroup(['D', 4]).codegrees()
                (0, 2, 2, 4)
                sage: CoxeterGroup(['F', 4]).codegrees()
                (0, 4, 6, 10)
                sage: CoxeterGroup(['E', 8]).codegrees()
                (0, 6, 10, 12, 16, 18, 22, 28)
                sage: CoxeterGroup(['H', 3]).codegrees()
                (0, 4, 8)

                sage: WeylGroup([["A",3], ["A",3], ["B",2]]).codegrees()
                (0, 1, 2, 0, 1, 2, 0, 2)
            """
            return tuple(d - 2 for d in self.degrees())

        @cached_method
        def weak_poset(self, side="right", facade=False):
            """
            INPUT:

            - ``side`` -- "left", "right", or "twosided" (default: "right")
            - ``facade`` -- a boolean (default: ``False``)

            Returns the left (resp. right) poset for weak order.  In
            this poset, `u` is smaller than `v` if some reduced word
            of `u` is a right (resp. left) factor of some reduced word
            of `v`.

            .. SEEALSO::

                :meth:`bhz_poset`, :meth:`bruhat_poset`, :meth:`shard_poset`

            EXAMPLES::

                sage: W = WeylGroup(["A", 2])
                sage: P = W.weak_poset()
                sage: P
                Finite lattice containing 6 elements
                sage: P.show()

            This poset is in fact a lattice::

                sage: W = WeylGroup(["B", 3])
                sage: P = W.weak_poset(side = "left")
                sage: P.is_lattice()
                True

            so this method has an alias :meth:`weak_lattice`::

                sage: W.weak_lattice(side = "left") is W.weak_poset(side = "left")
                True

            As a bonus feature, one can create the left-right weak
            poset::

                sage: W = WeylGroup(["A",2])
                sage: P = W.weak_poset(side = "twosided")
                sage: P.show()
                sage: len(P.hasse_diagram().edges(sort=False))
                8

            This is the transitive closure of the union of left and
            right order. In this poset, `u` is smaller than `v` if
            some reduced word of `u` is a factor of some reduced word
            of `v`. Note that this is not a lattice::

                sage: P.is_lattice()
                False

            By default, the elements of `P` are aware of that they
            belong to `P`::

                sage: P.an_element().parent()
                Finite poset containing 6 elements

            If instead one wants the elements to be plain elements of
            the Coxeter group, one can use the ``facade`` option::

                sage: P = W.weak_poset(facade = True)
                sage: P.an_element().parent()
                Weyl Group of type ['A', 2] (as a matrix group acting on the ambient space)

            .. SEEALSO:: :func:`Poset` for more on posets and facade posets.

            TESTS::

                sage: [len(WeylGroup(["A", n]).weak_poset(side = "right").cover_relations()) for n in [1,2,3]]
                [1, 6, 36]
                sage: [len(WeylGroup(["A", n]).weak_poset(side = "left" ).cover_relations()) for n in [1,2,3]]
                [1, 6, 36]

            .. TODO::

                - Use the symmetric group in the examples (for nicer
                  output), and print the edges for a stronger test.
                - The constructed poset should be lazy, in order to
                  handle large / infinite Coxeter groups.
            """
            from sage.combinat.posets.posets import Poset
            from sage.combinat.posets.lattices import LatticePoset
            if side == "twosided":
                covers = tuple([u, v] for u in self for v in u.upper_covers(side="left") + u.upper_covers(side="right"))
                return Poset((self, covers), cover_relations=True,
                             facade=facade)
            covers = tuple([u, v] for u in self for v in u.upper_covers(side=side))
            return LatticePoset((self, covers), cover_relations=True,
                                facade=facade)

        weak_lattice = weak_poset

        def inversion_sequence(self, word):
            r"""
            Return the inversion sequence corresponding to the ``word``
            in indices of simple generators of ``self``.

            If ``word`` corresponds to `[w_0,w_1,...w_k]`, the output is
            `[w_0,w_0w_1w_0,\ldots,w_0w_1\cdots w_k \cdots w_1 w_0]`.

            INPUT:

            - ``word`` -- a word in the indices of the simple
              generators of ``self``.

            EXAMPLES::

                sage: CoxeterGroup(["A", 2]).inversion_sequence([1,2,1])
                [
                [-1  1]  [ 0 -1]  [ 1  0]
                [ 0  1], [-1  0], [ 1 -1]
                ]

                sage: [t.reduced_word() for t in CoxeterGroup(["A",3]).inversion_sequence([2,1,3,2,1,3])]
                [[2], [1, 2, 1], [2, 3, 2], [1, 2, 3, 2, 1], [3], [1]]

            """
            return [self.from_reduced_word(word[:i+1]+list(reversed(word[:i])))
                    for i in range(len(word))]

        def reflections_from_w0(self):
            """
            Return the reflections of ``self`` using the inversion set
            of ``w_0``.

            EXAMPLES::

                sage: WeylGroup(['A',2]).reflections_from_w0()
                [
                [0 1 0]  [0 0 1]  [1 0 0]
                [1 0 0]  [0 1 0]  [0 0 1]
                [0 0 1], [1 0 0], [0 1 0]
                ]

                sage: WeylGroup(['A',3]).reflections_from_w0()
                [
                [0 1 0 0]  [0 0 1 0]  [1 0 0 0]  [0 0 0 1]  [1 0 0 0]  [1 0 0 0]
                [1 0 0 0]  [0 1 0 0]  [0 0 1 0]  [0 1 0 0]  [0 0 0 1]  [0 1 0 0]
                [0 0 1 0]  [1 0 0 0]  [0 1 0 0]  [0 0 1 0]  [0 0 1 0]  [0 0 0 1]
                [0 0 0 1], [0 0 0 1], [0 0 0 1], [1 0 0 0], [0 1 0 0], [0 0 1 0]
                ]
            """
            return self.long_element().inversions_as_reflections()

        @cached_method
        def m_cambrian_lattice(self, c, m=1, on_roots=False):
            """
            Return the `m`-Cambrian lattice on `m`-delta sequences.

            See :arxiv:`1503.00710` and :arxiv:`math/0611106`.

            The `m`-delta sequences are certain `m`-colored minimal
            factorizations of `c` into reflections.

            INPUT:

            - `c` -- a Coxeter element of ``self`` (as a tuple, or
              as an element of ``self``)

            - `m` -- a positive integer (default: 1)

            - ``on_roots`` (default: ``False``) -- if
              ``on_roots`` is ``True``, the lattice is realized on
              roots rather than on reflections. In order for this to
              work, the ElementMethod ``reflection_to_root`` must be
              available.

            EXAMPLES::

                sage: CoxeterGroup(["A",2]).m_cambrian_lattice((1,2))
                Finite lattice containing 5 elements

                sage: CoxeterGroup(["A",2]).m_cambrian_lattice((1,2),2)
                Finite lattice containing 12 elements
            """
            from sage.combinat.posets.lattices import LatticePoset
            if hasattr(c, "reduced_word"):
                c = c.reduced_word()
            c = list(c)

            sorting_word = self.long_element().coxeter_sorting_word(c)

            if on_roots:
                if not hasattr(self.long_element(), "reflection_to_root"):
                    raise ValueError("The parameter 'on_root=True' needs "
                                     "the ElementMethod 'reflection_to_root'")

                inv_woc = [t.reflection_to_root()
                           for t in self.inversion_sequence(sorting_word)]
                S = [s.reflection_to_root() for s in self.simple_reflections()]
                PhiP = [t.reflection_to_root() for t in self.reflections()]
            else:
                inv_woc = self.inversion_sequence(sorting_word)
                S = self.simple_reflections()
                T = self.reflections_from_w0()
                Twords = {t: t.reduced_word() for t in T}

            elements = set()
            covers = []

            bottom_elt = frozenset((s, 0) for s in S)
            new = {bottom_elt}
            while new:
                new_element = new.pop()
                elements.add(new_element)
                for t in new_element:
                    if t[1] < m:
                        cov_element = [s for s in new_element if s != t]
                        cov_element.append((t[0], t[1] + 1))
                        idx_t0 = inv_woc.index(t[0])
                        for t_conj in [(i, t[1]) for i in inv_woc[idx_t0:]] + [(i, t[1] + 1) for i in inv_woc[:idx_t0]]:
                            if t_conj in cov_element:
                                cov_element.remove(t_conj)
                                if on_roots:
                                    tmp = t_conj[0].weyl_action(t[0].associated_reflection())
                                    if tmp in PhiP:
                                        cov_element.append((tmp, t_conj[1]))
                                    else:
                                        cov_element.append((-tmp, t_conj[1] - 1))
                                else:
                                    tmp = t[0] * t_conj[0] * t[0]
                                    invs = self.inversion_sequence(Twords[t[0]]+Twords[t_conj[0]])
                                    plus_or_minus = invs.count(tmp)
                                    if plus_or_minus % 2:
                                        cov_element.append((tmp, t_conj[1]))
                                    else:
                                        cov_element.append((tmp, t_conj[1] - 1))

                        cov_element = frozenset(cov_element)
                        if cov_element not in elements:
                            new.add(cov_element)
                        covers.append((new_element, cov_element))
            return LatticePoset([elements, covers], cover_relations=True)

        def cambrian_lattice(self, c, on_roots=False):
            """
            Return the `c`-Cambrian lattice on delta sequences.

            See :arxiv:`1503.00710` and :arxiv:`math/0611106`.

            Delta sequences are certain 2-colored minimal factorizations
            of ``c`` into reflections.

            INPUT:

            - ``c`` -- a standard Coxeter element in ``self``
              (as a tuple, or as an element of ``self``)

            - ``on_roots`` (default: ``False``) -- if
              ``on_roots`` is ``True``, the lattice is realized on
              roots rather than on reflections. In order for this to
              work, the ElementMethod ``reflection_to_root`` must be
              available.

            EXAMPLES::

                sage: CoxeterGroup(["A", 2]).cambrian_lattice((1,2))
                Finite lattice containing 5 elements

                sage: CoxeterGroup(["B", 2]).cambrian_lattice((1,2))
                Finite lattice containing 6 elements

                sage: CoxeterGroup(["G", 2]).cambrian_lattice((1,2))
                Finite lattice containing 8 elements
            """
            return self.m_cambrian_lattice(c=c, m=1, on_roots=on_roots)

        def is_real(self):
            """
            Return ``True`` since ``self`` is a real reflection group.

            EXAMPLES::

                sage: CoxeterGroup(['F',4]).is_real()
                True
                sage: CoxeterGroup(['H',4]).is_real()
                True
            """
            return True

        def permutahedron(self, point=None, base_ring=None):
            r"""
            Return the permutahedron of ``self``,

            This is the convex hull of the point ``point`` in the weight
            basis under the action of ``self`` on the underlying vector
            space `V`.

            .. SEEALSO::

                :meth:`~sage.combinat.root_system.reflection_group_real.permutahedron`

            INPUT:

            - ``point`` -- optional, a point given by its coordinates in
              the weight basis (default is `(1, 1, 1, \ldots)`)
            - ``base_ring`` -- optional, the base ring of the polytope

            .. NOTE::

                The result is expressed in the root basis coordinates.

            .. NOTE::

                If function is too slow, switching the base ring to
                :class:`RDF` will almost certainly speed things up.

            EXAMPLES::

                sage: W = CoxeterGroup(['H',3], base_ring=RDF)
                sage: W.permutahedron()
                doctest:warning
                ...
                UserWarning: This polyhedron data is numerically complicated; cdd could not convert between the inexact V and H representation without loss of data. The resulting object might show inconsistencies.
                A 3-dimensional polyhedron in RDF^3 defined as the convex hull of 120 vertices

                sage: W = CoxeterGroup(['I',7])
                sage: W.permutahedron()
                A 2-dimensional polyhedron in AA^2 defined as the convex hull of 14 vertices
                sage: W.permutahedron(base_ring=RDF)
                A 2-dimensional polyhedron in RDF^2 defined as the convex hull of 14 vertices

                sage: W = ReflectionGroup(['A',3])                          # optional - gap3
                sage: W.permutahedron()                                     # optional - gap3
                A 3-dimensional polyhedron in QQ^3 defined as the convex hull
                of 24 vertices

                sage: W = ReflectionGroup(['A',3],['B',2])                  # optional - gap3
                sage: W.permutahedron()                                     # optional - gap3
                A 5-dimensional polyhedron in QQ^5 defined as the convex hull of 192 vertices

            TESTS::

                sage: W = ReflectionGroup(['A',3])                          # optional - gap3
                sage: W.permutahedron([3,5,8])                              # optional - gap3
                A 3-dimensional polyhedron in QQ^3 defined as the convex hull
                of 24 vertices


            .. PLOT::
                :width: 300 px

                W = CoxeterGroup(['I',7])
                p = W.permutahedron()
                sphinx_plot(p)

            """
            n = self.one().canonical_matrix().rank()
            weights = self.fundamental_weights()
            if point is None:
                from sage.rings.integer_ring import ZZ
                point = [ZZ.one()] * n
            v = sum(point[i-1] * weights[i] for i in weights.keys())
            vertices = [v*w for w in self]
            if base_ring is None:
                if isinstance(v.base_ring(), (sage.rings.abc.UniversalCyclotomicField,
                                              sage.rings.abc.AlgebraicField_common)):
                    from sage.rings.qqbar import AA
                    vertices = [v.change_ring(AA) for v in vertices]
                    base_ring = AA
            from sage.geometry.polyhedron.constructor import Polyhedron
            return Polyhedron(vertices=vertices, base_ring=base_ring)

        def coxeter_poset(self):
            r"""
            Return the Coxeter poset of ``self``.

            Let `W` be a Coxeter group. The *Coxeter poset* is defined as
            the set of (right) standard cosets `gW_J`, where `J` is a
            subset of the index set `I` of `W`, ordered by reverse inclusion.

            This is equal to the face poset of the :meth:`Coxeter complex
            <coxeter_complex()>`.

            EXAMPLES::

                sage: W = CoxeterGroup(['A', 3])
                sage: P = W.coxeter_poset()
                sage: P
                Finite meet-semilattice containing 75 elements
                sage: P.rank()
                3

                sage: W = WeylGroup(['B', 3])
                sage: P = W.coxeter_poset()
                sage: P
                Finite meet-semilattice containing 147 elements
                sage: P.rank()
                3

                sage: W = CoxeterGroup(['I', 7])
                sage: P = W.coxeter_poset()
                sage: P
                Finite meet-semilattice containing 29 elements
                sage: P.rank()
                2

                sage: W = CoxeterGroup(['H', 3])
                sage: P = W.coxeter_poset()
                sage: P
                Finite meet-semilattice containing 363 elements
                sage: P.rank()
                3

                sage: # optional - gap3
                sage: W = CoxeterGroup(['H', 3], implementation="permutation")
                sage: P = W.coxeter_poset()
                sage: P
                Finite meet-semilattice containing 363 elements
                sage: P.rank()
                3
            """
            I = self.index_set()
            data = {}
            next_level = {(g, ()) for g in self}
            while next_level:
                cur = next_level
                next_level = set()
                for Y in cur:
                    g, J = Y
                    for i in I:
                        if i in J:
                            continue
                        Jp = tuple(sorted(J + (i,)))
                        gp = g.coset_representative(Jp, side='right')
                        X = (gp, Jp)
                        if X in data:
                            data[X].append(Y)
                        else:
                            data[X] = [Y]
                        next_level.add(X)
            from sage.combinat.posets.lattices import MeetSemilattice
            return MeetSemilattice(data)

        def coxeter_complex(self):
            r"""
            Return the Coxeter complex of ``self``.

            Let `W` be a Coxeter group, and let `X` be the corresponding Tits
            cone, which is constructed as the `W` orbit of the fundamental
            chamber in the reflection representation. The *Coxeter complex*
            of `W` is the simplicial complex `(X \setminus \{0\}) / \RR_{>0}`.
            The face poset of this simplicial complex is given by the
            :meth:`coxeter_poset()`. When `W` is a finite group, then the
            Coxeter complex is homeomorphic to an `(n-1)`-dimensional sphere,
            where `n` is the rank of `W`.

            EXAMPLES::

                sage: W = CoxeterGroup(['A', 3])
                sage: C = W.coxeter_complex()
                sage: C
                Simplicial complex with 14 vertices and 24 facets
                sage: C.homology()
                {0: 0, 1: 0, 2: Z}

                sage: W = WeylGroup(['B', 3])
                sage: C = W.coxeter_complex()
                sage: C
                Simplicial complex with 26 vertices and 48 facets
                sage: C.homology()
                {0: 0, 1: 0, 2: Z}

                sage: W = CoxeterGroup(['I', 7])
                sage: C = W.coxeter_complex()
                sage: C
                Simplicial complex with 14 vertices and 14 facets
                sage: C.homology()
                {0: 0, 1: Z}

                sage: W = CoxeterGroup(['H', 3])
                sage: C = W.coxeter_complex()
                sage: C
                Simplicial complex with 62 vertices and 120 facets
                sage: C.homology()
                {0: 0, 1: 0, 2: Z}

                sage: # optional - gap3
                sage: W = CoxeterGroup(['H', 3], implementation="permutation")
                sage: C = W.coxeter_complex()
                sage: C
                Simplicial complex with 62 vertices and 120 facets
                sage: C.homology()
                {0: 0, 1: 0, 2: Z}
            """
            I = self.index_set()
            facets = {}
            Ip = {i: tuple([j for j in I if j != i]) for i in I}
            for g in self:
                V = []
                for i in I:
                    gp = g
                    D = gp.descents(side='right')
                    if D and D[0] == i:
                        D.pop(0)
                    while D:
                        gp = gp.apply_simple_reflection(D[0])
                        D = gp.descents(side='right')
                        if D and D[0] == i:
                            D.pop(0)
                    V.append((gp, Ip[i]))
                facets[g] = V
            verts = set()
            for F in facets.values():
                verts.update(F)
            labels = {x: i for i, x in enumerate(verts)}
            result = [[labels[v] for v in F] for F in facets.values()]
            from sage.topology.simplicial_complex import SimplicialComplex
            return SimplicialComplex(result)

    class ElementMethods:

        def absolute_length(self):
            """
            Return the absolute length of ``self``.

            The absolute length is the length of the shortest expression
            of the element as a product of reflections. For finite Coxeter
            groups, the absolute length is the codimension of the
            1-eigenspace of the element (Lemmas 1-3 in [Car1972a]_).

            For permutations in the symmetric groups, the absolute
            length is the size minus the number of its disjoint
            cycles.

            .. SEEALSO::

                :meth:`~sage.categories.coxeter_groups.absolute_le`

            EXAMPLES::

                sage: W = WeylGroup(["A", 3])                                           # needs sage.combinat sage.groups
                sage: s = W.simple_reflections()                                        # needs sage.combinat sage.groups
                sage: (s[1]*s[2]*s[3]).absolute_length()                                # needs sage.combinat sage.groups
                3

                sage: W = SymmetricGroup(4)                                             # needs sage.groups
                sage: s = W.simple_reflections()                                        # needs sage.groups
                sage: (s[3]*s[2]*s[1]).absolute_length()                                # needs sage.combinat sage.groups
                3
            """
            M = self.canonical_matrix()
            return (M - 1).image().dimension()

        @cached_in_parent_method
        def bruhat_upper_covers(self):
            r"""
            Returns all the elements that cover ``self`` in Bruhat order.

            EXAMPLES::

                sage: W = WeylGroup(["A",4])
                sage: w = W.from_reduced_word([3,2])
                sage: print([v.reduced_word() for v in w.bruhat_upper_covers()])
                [[4, 3, 2], [3, 4, 2], [2, 3, 2], [3, 1, 2], [3, 2, 1]]

                sage: W = WeylGroup(["B",6])
                sage: w = W.from_reduced_word([1,2,1,4,5])
                sage: C = w.bruhat_upper_covers()
                sage: len(C)
                9
                sage: print([v.reduced_word() for v in C])
                [[6, 4, 5, 1, 2, 1], [4, 5, 6, 1, 2, 1], [3, 4, 5, 1, 2, 1], [2, 3, 4, 5, 1, 2],
                [1, 2, 3, 4, 5, 1], [4, 5, 4, 1, 2, 1], [4, 5, 3, 1, 2, 1], [4, 5, 2, 3, 1, 2],
                [4, 5, 1, 2, 3, 1]]
                sage: ww = W.from_reduced_word([5,6,5])
                sage: CC = ww.bruhat_upper_covers()
                sage: print([v.reduced_word() for v in CC])
                [[6, 5, 6, 5], [4, 5, 6, 5], [5, 6, 4, 5], [5, 6, 5, 4], [5, 6, 5, 3], [5, 6, 5, 2],
                [5, 6, 5, 1]]

            Recursive algorithm: write `w` for ``self``. If `i` is a
            non-descent of `w`, then the covers of `w` are exactly
            `\{ws_i, u_1s_i, u_2s_i,..., u_js_i\}`, where the `u_k`
            are those covers of `ws_i` that have a descent at `i`.
            """

            i = self.first_descent(positive=True, side='right')
            if i is not None:
                wsi = self.apply_simple_reflection(i, side='right')
                return [u.apply_simple_reflection(i, side='right')
                        for u in wsi.bruhat_upper_covers()
                        if u.has_descent(i, side='right')] + [wsi]
            return []

        def coxeter_knuth_neighbor(self, w):
            r"""
            Return the Coxeter-Knuth (oriented) neighbors of the reduced word `w` of ``self``.

            INPUT:

            - ``w`` -- reduced word of ``self``

            The Coxeter-Knuth relations are given by `a a+1 a \sim a+1 a a+1`, `abc \sim acb`
            if `b<a<c` and `abc \sim bac` if `a<c<b`. This method returns all neighbors of
            ``w`` under the Coxeter-Knuth relations oriented from left to right.

            EXAMPLES::

                sage: W = WeylGroup(['A',4], prefix='s')
                sage: word = [1,2,1,3,2]
                sage: w = W.from_reduced_word(word)
                sage: w.coxeter_knuth_neighbor(word)
                {(1, 2, 3, 1, 2), (2, 1, 2, 3, 2)}

                sage: word = [1,2,1,3,2,4,3]
                sage: w = W.from_reduced_word(word)
                sage: w.coxeter_knuth_neighbor(word)
                {(1, 2, 1, 3, 4, 2, 3), (1, 2, 3, 1, 2, 4, 3), (2, 1, 2, 3, 2, 4, 3)}

            TESTS::

                sage: W = WeylGroup(['B',4], prefix='s')
                sage: word = [1,2]
                sage: w = W.from_reduced_word(word)
                sage: w.coxeter_knuth_neighbor(word)
                Traceback (most recent call last):
                ...
                NotImplementedError: this has only been implemented in finite type A so far
            """
            C = self.parent().cartan_type()
            if not C[0] == 'A':
                raise NotImplementedError("this has only been implemented in finite type A so far")
            d = []
            for i in range(2, len(w)):
                v = list(w)
                if w[i-2] == w[i]:
                    if w[i] == w[i-1] - 1:
                        v[i-2] = w[i-1]
                        v[i] = w[i-1]
                        v[i-1] = w[i]
                        d += [tuple(v)]
                elif w[i-1] < w[i-2] and w[i-2] < w[i]:
                    v[i] = w[i-1]
                    v[i-1] = w[i]
                    d += [tuple(v)]
                elif w[i-2] < w[i] and w[i] < w[i-1]:
                    v[i-2] = w[i-1]
                    v[i-1] = w[i-2]
                    d += [tuple(v)]
            return set(d)

        def coxeter_knuth_graph(self):
            r"""
            Return the Coxeter-Knuth graph of type `A`.

            The Coxeter-Knuth graph of type `A` is generated by the Coxeter-Knuth relations which are
            given by `a a+1 a \sim a+1 a a+1`, `abc \sim acb` if `b<a<c` and `abc \sim bac` if `a<c<b`.

            EXAMPLES::

                sage: W = WeylGroup(['A',4], prefix='s')
                sage: w = W.from_reduced_word([1,2,1,3,2])
                sage: D = w.coxeter_knuth_graph()
                sage: D.vertices(sort=True)
                [(1, 2, 1, 3, 2),
                (1, 2, 3, 1, 2),
                (2, 1, 2, 3, 2),
                (2, 1, 3, 2, 3),
                (2, 3, 1, 2, 3)]
                sage: D.edges(sort=True)
                [((1, 2, 1, 3, 2), (1, 2, 3, 1, 2), None),
                ((1, 2, 1, 3, 2), (2, 1, 2, 3, 2), None),
                ((2, 1, 2, 3, 2), (2, 1, 3, 2, 3), None),
                ((2, 1, 3, 2, 3), (2, 3, 1, 2, 3), None)]

                sage: w = W.from_reduced_word([1,3])
                sage: D = w.coxeter_knuth_graph()
                sage: D.vertices(sort=True)
                [(1, 3), (3, 1)]
                sage: D.edges(sort=False)
                []

            TESTS::

                sage: W = WeylGroup(['B',4], prefix='s')
                sage: w = W.from_reduced_word([1,2])
                sage: w.coxeter_knuth_graph()
                Traceback (most recent call last):
                ...
                NotImplementedError: this has only been implemented in finite type A so far
            """
            from sage.graphs.graph import Graph
            R = [tuple(v) for v in self.reduced_words()]
            G = Graph()
            G.add_vertices(R)
            G.add_edges([v, vp] for v in R for vp in self.coxeter_knuth_neighbor(v))
            return G

        def is_coxeter_element(self):
            r"""
            Return whether this is a Coxeter element.

            This is, whether ``self`` has an eigenvalue `e^{2\pi i/h}`
            where `h` is the Coxeter number.

            .. SEEALSO:: :meth:`~sage.categories.finite_complex_reflection_groups.coxeter_elements`

            EXAMPLES::

                sage: W = CoxeterGroup(['A',2])
                sage: c = prod(W.gens())
                sage: c.is_coxeter_element()
                True
                sage: W.one().is_coxeter_element()
                False

                sage: W = WeylGroup(['G', 2])
                sage: c = prod(W.gens())
                sage: c.is_coxeter_element()
                True
                sage: W.one().is_coxeter_element()
                False
            """
            return self in self.parent().coxeter_elements()

        def covered_reflections_subgroup(self):
            """
            Return the subgroup of `W` generated by the conjugates by `w`
            of the simple reflections indexed by right descents of `w`.

            This is used to compute the shard intersection order on `W`.

            EXAMPLES::

                sage: W = CoxeterGroup(['A',3], base_ring=ZZ)
                sage: len(W.long_element().covered_reflections_subgroup())
                24
                sage: s = W.simple_reflection(1)
                sage: Gs = s.covered_reflections_subgroup()
                sage: len(Gs)
                2
                sage: s in [u.lift() for u in Gs]
                True
                sage: len(W.one().covered_reflections_subgroup())
                1
            """
            W = self.parent()
            winv = ~self
            cov_down = [self * W.simple_reflection(i) * winv
                        for i in self.descents(side='right')]
            return W.submonoid(cov_down)
